
/**************************************************************************/ 
/*                                                                        */ 
/*       Copyright (c) 2006 by Control Technology Corporation             */ 
/*                                                                        */ 
/*  This software is copyrighted by and is the sole property of Control   */ 
/*  Technology Corporation.  All rights,title,ownership,or other interests*/ 
/*  in the software remain the property of Control Technology Corporation */ 
/*  Any unauthorized use, duplication, transmission,  distribution,       */ 
/*  or disclosure of this software is expressly forbidden.                */ 
/*                                                                        */
/*  This Copyright notice may not be removed or modified without prior    */ 
/*  written consent of Control Technology Corporation                     */ 
/*                                                                        */ 
/*  Control Technology Corporation reserves the right to modify this      */ 						
/*  software without notice.                                              */ 
/*                                                                        */ 
/*  Control Technology Corporation          info@ctc-control.com          */
/*  25 South Street                         http://www.ctc-control.com    */
/*  Hopkinton, MA  01748                                                  */
/*                                                                        */
/**************************************************************************/

/**************************************************************************/
/**                                                                       */ 
/** Variant Register Definition Component                                 */
/**                                                                       */
/**   ARM9 Initialization and generic backplane interface routines        */ 
/**                                                                       */ 
/**************************************************************************/

/**************************************************************************/ 
/*                                                                        */ 
/*  COMPONENT DEFINITION                                                  */ 
/*                                                                        */ 
/*    variant.h                                                           */ 
/*                                                                        */ 
/*  AUTHOR                                                                */ 
/*                                                                        */ 
/*    Kevin Halloran, Control Technology Corporation                      */ 
/*                                                                        */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This file contains the definitions required for a Variant Register  */
/*    type within the Quickstep Language                                  */
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/*  02-17-2006      Kevin Halloran           Initial Development          */ 
/*  06-02-2006      Kevin Halloran           Updated for QS4/UserC        */ 
/**************************************************************************/ 

#ifndef VARIANT_H
#define VARIANT_H

#define VARIANT_MAX_STRING 223	// maximum size of user string storage (add 1 for array null)

// Flags
#define VARIANT_QS4TABLE_NAME_FLAG BIT0	// if set the name of the property is read
//#define VARIANT_INDIRECTION_FLAG BIT6  // setting this bit means that the integer portion of
									   // this is to be used to access another register's value


// Bit identifiers for settings and last written variant type
#define VARIANT_INTEGER BIT0
#define VARIANT_UINTEGER BIT1
#define VARIANT_STRING BIT2
#define VARIANT_FLOAT BIT3
#define VARIANT_DOUBLE BIT4
#define VARIANT_STRING_PTR BIT5
#define VARIANT_INDIRECTION_FLAG BIT6  // setting this bit means that the integer portion of
									   // this is to be used to access another register's value
#define VARIANT_NONVOLATILE BIT7   // storage structure is nonvolatile memory

// Requests for Variant access types in addition to VARIANT_INTEGER to VARIANT_DOUBLE
#define VARIANT_BASE_PROPERTIES_TYPE BIT10  // Get properties only
#define VARIANT_STORAGE_TYPE BIT11  // This is for requests of read/write not settings
#define VARIANT_BLOCKACCESS BIT12	// block access of data, VARIANT_STORAGE_BLOCK used instead of VARIANT_STORAGE, both bits should be set


#define VARIANT_CURRENT_TYPE -1		// Used as flag with VARIANT_STORAGE_TYPE to get current Variant in
									// its native (current) storage format/type

// COMMANDS
#define VARIANT_CMD_NULL 0			// Normal access
#define VARIANT_CMD_INDIRECTION 1	// Assumes that a VARIANT_INTEGER will be written such that a
									// nonzero value sets the indirection bit
#define VARIANT_CMD_CLEAR 2			// This command clears the variant to type integer (0), deleting array, deletes NV variant.
#define VARIANT_CMD_SET_IFNEEDED 3	// This command only effects NV variants.  Basically should be treated as a write operation.
									// If the X dimension of the array is indexX+1 then nothing occurs, if not array is deleted
									// and recreated writing the passed value to the indicated indexX/indexY cell.


// Default precision for floats to string conversion
#define DEFAULT_VARIANT_FLOAT_PRECISION 6

// Maximum array depth allowed, note that the MAX is not possible if using both indexes, also it will run SLOW given each
// element is in a linked list
#define VARIANT_ARRAY_MAX_COL_ELEMENTS 32768
#define VARIANT_ARRAY_MAX_ROW_ELEMENTS 32768

// Mode of operation
#define VARIANT_READ 0
#define VARIANT_WRITE 1

// Local registers
#define VARIANT_REG_NUMBER_LOCAL   100
#define VARIANT_REG_LOCAL_FIRST         RegisterNumber( 36001)
#define VARIANT_REG_LOCAL_LAST          RegisterNumber( 36100)

// Public Registers
#define VARIANT_REG_NUMBER_VOLATILE	 600
#define VARIANT_REG_VPUBLIC_FIRST         RegisterNumber( 36101)
#define VARIANT_REG_VPUBLIC_LAST          RegisterNumber( 36700)

// NonVolatile registers
#define VARIANT_REG_NUMBER_NVOLATILE 100
#define VARIANT_REG_NVPUBLIC_FIRST         RegisterNumber( 36701)
#define VARIANT_REG_NVPUBLIC_LAST          RegisterNumber( 36800)

/*
Variant property selection registers

All Variant registers may become arrays by setting the 'Size' property a particular variant.  
Variant register property access is via a Selection register, local to all tasks.

Variant Selection Register - Register 36804.  Pointer to a Variant register (36001 to 36800) 
                             whose property to access.
Variant Index Register -     Register 36807.  Index value of Variant selected, base is 0.
Variant Size Register -      Register 36808.  Size of Variant array, write changes size, default is 1.
Variant Increment Register - Register 36809.  0 to disable, 1 to increment, and -1 to decrement.  
                             Index cleared to 0 on write.  Each read or write to the actual Variant 
                             Register will increment or decrement the index until it can access no 
                             more at which point a fault will occur if bounds exceeded.
Variant Deletion Register -  Register 36810.  Allows the specified Variant to be deleted
NonVolatile Close Register - Register 36811.  Closes a nonvolatile variant file.
*/

#define REG_LOCAL_VARIANT_SELECTION	    RegisterNumber( 36804)		// read/write
#define REG_LOCAL_VARIANT_ARRAYSIZE_COL	RegisterNumber( 36805)		// read/write
#define REG_LOCAL_VARIANT_ARRAYSIZE_ROW	RegisterNumber( 36806)		// read/write
#define REG_LOCAL_VARIANT_INDEX_COL		RegisterNumber( 36807)		// read/write
#define REG_LOCAL_VARIANT_INDEX_ROW		RegisterNumber( 36808)		// read/write
#define REG_LOCAL_VARIANT_INDIRECTION	RegisterNumber( 36809)	   	// read/write
#define REG_VARIANT_DELETION			RegisterNumber( 36810)	   	// write
#define REG_NVARIANT_CLOSE				RegisterNumber( 36811)	   	// write

#define REG_QS4OBJECT_TABLE_VARIANT		RegisterNumber( 36820)		// Generic Variant table access
#define REG_QS4OBJECT_TABLE_VARIANTPROP_NAME		RegisterNumber( 36821)		// Generic Variant table access QS4 Object property name

/*
Task Control Lock Register

Writing any value to this register will cause the calling TASK to maintain control for
an additional step.
*/
#define REG_LOCAL_TASK_CONTROL_LOCK RegisterNumber (36996)

/*
QS4 General resource filter use:
*/
#define REG_QS4_WRITE_FUNCTION_CALL RegisterNumber (36990)
#define REG_QS4_READ_FUNCTION_CALL RegisterNumber (36991)

typedef struct varHeader
{
	struct varHeader *nextCol;
	struct varHeader *nextRow;
	int iValue;
	float fValue;
	double dValue;
	unsigned char stringLen;  // length of string stored
	unsigned char settings;   // Bit 0 - int valid, 1 - string, 2 - float, 4 - double, 
							  // 6 - indirection, 7 - nonvolatile 
	unsigned char lastWrite;  // Bit 0 - int, 1 - string, 2 - float, 4 - double
	unsigned char precision;  // double to string conversion precision %.6f default 
	short mallocLen;  // Length of space reserved for the string 
	unsigned short indexCol;	  // Col dimension index reference
	unsigned short indexRow;	  // Row dimension index reference
	unsigned short arraysizeCols;    // depth of array in Col dimension, default is 1.
	unsigned short arraysizeRows;    // depth of array in Row dimension, default is 1.
} VAR_HEADER;

// Volatile Variant register
typedef struct
{
	VAR_HEADER header;
	char *sValue;
} VOL_VARIANT_REGISTER;

#define MAX_VARIANT_CACHE_ENTRIES 1024
typedef struct varCache
{
	struct varCache *next;	// pointer to next variant pointer block
	int offset;				// where this starts in the food chain, 0 is first
	int entriesMax;			// number of slots present
	VOL_VARIANT_REGISTER *vptr[1];	// pointer to variant entry	
} VARIANT_CACHE_TABLE;
		
// Variant properties
typedef struct
{
	unsigned short indexCol;	  // Col dimension index reference
	unsigned short indexRow;	  // Row dimension index reference
	unsigned short arraysizeCols;	// depth of array in Col dimension, default is 1.
	unsigned short arraysizeRows;	// depth of array in Row dimension, default is 1.
} VARIANT_BASE_PROPERTIES;

// Non-volatile Variant register
typedef struct
{
	VAR_HEADER header;
	unsigned long sValue;	// file offset to string to use
	short openIdx;	// open file index using
	unsigned short nvIndex;  // Which NV register this is, 0 base
	unsigned int cksum;		// checksum for storage verification, make int for alignment purposes
} NV_VARIANT_REGISTER;  // stored in battery backed RAM.

// Universal variant storage structure, aligned on 4 byte boundry
typedef struct
{
	int type;					// type of storage being used or requested
								// If -1 on read then return current, else set to type want.
								// On write must set to type that is stored within this structure
	unsigned char precision;  	// double to string conversion precision %.6f default
								// On read is what is presently set, write what want. 
	unsigned char flags;		// special flags for processing so far only
								// VARIANT_INDIRECTION_FLAG used, can be used to set property
								// in ->settings on write operation, no effect on read.  Written
								// value becomes register to reference for further operations.
	unsigned char cmd;			// 00, no operation other than read/write specified, else do defined
								// operation.  Currently have write for properties access to 'settings'
								// VARIANT_CMD_SET_INDIRECTION and VARIANT_CMD_CLEAR_INDIRECTION, 
								// write value ignored.
	unsigned char pad;
    unsigned short taskHandle; // task number (offset in task array + 1, where 0 is 1) or handle thus usable from remote or C API, 4096 to 65535, set to 0 for public reg.
    unsigned short slength;    // this is reserved for later use and possible string length if want unsigned char, 0  255 values, VARIANT_BYTE, future type
	unsigned int indexCol;	  // Col dimension index reference, 0xffffffff for using default of current variant
	unsigned int indexRow;	  // Row dimension index reference
	union						// Data that was read or has been written of 'type'
	{
		int iValue;
		unsigned int uiValue;
		float fValue;
		double dValue;
		unsigned int dSwap[2];	// used to swap doubles for PC access
		char *psValue;
		char sValue[VARIANT_MAX_STRING+1];
	} data;
} VARIANT_STORAGE;

typedef struct
{
	int numAccess;	// number of items to access
	int rowInc;		// row increment, if 0 just read columns based upon colInc.
	int colInc;		// col increment, if 0 just increment rows.
	int arraysizeCols;	// Used on write operation, -1 do not expand existing columns, else columns desired.  Rows will automatically grow as needed
} BLOCKACCESS;

#define MAX_VARIANT_BLOCK_32BITS	346		// 346 integers
#define MAX_VARIANT_BLOCK_64BITS 	173 	// 173 doubles
// Allow for block reads
typedef struct
{
	int type;					// type of storage being used or requested
								// If -1 on read then return current, else set to type want.
								// On write must set to type that is stored within this structure
								// write not supported for block access
								// If block access type field will be 0 if error else type of first cell.
								// slength will be the number of elements returned within data.block.?[n]
	unsigned char precision;  	// double to string conversion precision %.6f default
								// On read is what is presently set, write what want. 
	unsigned char flags;		// special flags for processing so far only
								// VARIANT_INDIRECTION_FLAG used, can be used to set property
								// in ->settings on write operation, no effect on read.  Written
								// value becomes register to reference for further operations.
	unsigned char cmd;			// 00, no operation other than read/write specified, else do defined
								// operation.  Currently have write for properties access to 'settings'
								// VARIANT_CMD_SET_INDIRECTION and VARIANT_CMD_CLEAR_INDIRECTION, 
								// write value ignored.
	unsigned char pad;
    unsigned short taskHandle; // task number (offset in task array + 1, where 0 is 1) or handle thus usable from remote or C API, 4096 to 65535, set to 0 for public reg.
    unsigned short slength;    // this is reserved for later use and possible string length if want unsigned char, 0  255 values, VARIANT_BYTE, future type
	unsigned int indexCol;	  // Col dimension index reference, 0xffffffff for using default of current variant
	unsigned int indexRow;	  // Row dimension index reference
	union						// Data that was read or has been written of 'type'
	{
		int iValue;
		unsigned int uiValue;
		float fValue;
		double dValue;
		unsigned int dSwap[2];	// used to swap doubles for PC access
		char *psValue;
		char sValue[VARIANT_MAX_STRING+1];
								// will be stored in same VARIANT_STORAGE upon return, thus data.blockread.numAccess * sizeof(variant type)
		// if BLOCKACCESS then iValue[n], fValue[n], or dValue[n]
		struct 
		{
			BLOCKACCESS blockaccess;	// Defines block read of variant cells, data storage must be big enough since
			union	  
			{
				int ibValue[MAX_VARIANT_BLOCK_32BITS];	   
				float fbValue[MAX_VARIANT_BLOCK_32BITS];   
				double dbValue[MAX_VARIANT_BLOCK_64BITS];  
			};
		} block;
	} data;
} VARIANT_STORAGE_BLOCK;

// TASK local register storage
typedef struct
{
	// local variant registers
	VOL_VARIANT_REGISTER *vol_variantreg[VARIANT_REG_NUMBER_LOCAL];  // local volatile registers
	// local control registers
	int variantSelectionRegister;
	int variantArraySizeRegister;
} TASK_LOCAL_REGS;

typedef struct
{
	unsigned int totCol;		// total X in entire file if first block.
	unsigned int totRow;		// total Y in entire file if first block.
	unsigned int entriesCol;	// Col entries in this table
	unsigned int entriesRow;	// Row entries in this table
	unsigned long nextBlock;	// next table block entry
} NV_BLOCK_HEADER;		// not currently used.

#define MAX_NVARIANT_FILES 10
typedef struct
{
	short index;		// index of variant in array this belongs to
	void *fileHandle;	// handle
	char mode[4];		// open properties
	NV_VARIANT_REGISTER *nvptr;	// pointer to nonvolatile variant
	unsigned long lastAccess;		// when last accessed
	NV_BLOCK_HEADER vheader;		// first block
	unsigned long lastVPosition;	// start of last variant read so can seek back to it
} NV_FILEHANDLE;

typedef struct
{
	const int* offsets;
	const int* types;
	const char** names;
	const char* name;
} QS4OBJECT_HEADER;

	
/* make sure that the header is processed correctly in a C++ module. */
#ifdef   __cplusplus
extern   "C" {
#endif
// Below functions not directly accessible for User 'C' API
RETVAL VariantRegister_Get( TASK *task, UINT16 RegNum, void *RegVal, int type);
RETVAL VariantRegister_Put( TASK *task, UINT16 RegNum, void *RegVal, int type); 
RETVAL VariantRegister_Init( void); 
RETVAL VariantRegister_PowerupReset( int mounted); 
RETVAL VariantRegister_Reset( void); 
RETVAL getVariantRegisterLock(UINT16 RegNum);
RETVAL releaseVariantRegisterLock(UINT16 RegNum);
int closeVariant(int reg, int deleteFlag);
void reloadNV_Variant(int regnum, int mounted);
VOL_VARIANT_REGISTER *getVariantRegister(TASK *task, int RegNum, int accessType, int lock);
void freeVariantVolatile(VOL_VARIANT_REGISTER *vptr);

/* End of C++ wrapper started above.  */
#ifdef   __cplusplus
        }
#endif

#endif  // VARIANT_H
