/*
    Quickstep Core Function

  This module contains the routines for the User C application
  to access the Quickstep Core Functions via the table

*/

#include "CoreFunc.h"

/*
***********************************************************
Local variables
***********************************************************
*/

const VTABLE_QS *coreTable;

#if defined( __arm__ )

extern unsigned long _user_load_point;

#define CODE_LOADADDRESS    (0x20100020L)
#define TABLE_PRESENT_VALUE (0xbeeffaceL)

const VTABLE_QS *getCoreAddress(void)
{
  unsigned long *pointer = (unsigned long *)CODE_LOADADDRESS;
	unsigned long address = NULL;

  // check that the constant indicating a valid address is present
  if (*pointer == TABLE_PRESENT_VALUE)
    {
    // constant is there, get the address and make sure it is in a valid range
    address = *(++pointer);
    if ((address > (unsigned long)pointer) &&
        (address < _user_load_point) )
      {
      // invalid address - force it to NULL
      address = (unsigned long)NULL;
      }
    }

	return((const VTABLE_QS *)address);
}


#elif defined( __sh__ )
#define COREBASE   63

const VTABLE_QS *getCoreAddress(void)
{
	register const VTABLE_QS *address;

	asm volatile (
		"trapa  %1;         "
		"mov    r0,%0; "
		: "=r" (address)      // output in address
		: "i" (COREBASE)      // input is trap number
	: "r0");              // uses r0

	return(address);
}

#else
#error "Architecture not supported"
#endif
int initCoreFunction(void)
{
	coreTable = getCoreAddress();
	if (coreTable == NULL)
		return -1;		// error
	else
		return 0;		// tis OK
}

const VTABLE_QS *getCoreTable(void)
{
	if (coreTable == NULL)
	{
		coreTable = getCoreAddress();
	}

	return(coreTable);
}

/************************* Register Access ********************************/
/*                                                                        */
/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    regRead                                             PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*    Read a Quickstep Register value.                                    */
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    UINT16 RegNum - Register number from 1 to 64535 to read             */ 
/*    INT32 *RegVal - Pointer to a 32 bit wide Integer to store the result*/ 
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*    RETVAL -                                                            */ 
/*      SUCCESS = function called properly                                */ 
/*      ERROR_NOT_DEFINED = Quickstep OS table not found                  */
/*      (also register specific return values defined in Errors.h)        */ 
/*      ERROR_NO_REG if invalid register                                  */
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    As required by user code                                            */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/*                                                                        */ 
/**************************************************************************/ 
RETVAL regRead(UINT16 RegNum, INT32 *RegVal)
{
	//	REGISTER_GET            regRead;
	if (getCoreTable() != NULL)
	{
		if ((coreTable->regRead != NULL) && (RegNum >=1))
		{
			RegNum--;	// make 0 based
			return((*coreTable->regRead)(RegNum, RegVal));
		}
	}
	return(ERROR_NOT_DEFINED);
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    regWrite                                            PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*    Write a value to a Quickstep Register                               */
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    UINT16 RegNum - Register number from 1 to 64536 to read             */ 
/*    INT32  RegVal - 32 bit wide Integer value to store                  */ 
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*    RETVAL -                                                            */ 
/*      SUCCESS = function called properly                                */ 
/*      ERROR_NOT_DEFINED = Quickstep OS table not found                  */
/*      (also register specific return values defined in Errors.h)        */ 
/*      ERROR_NO_REG if invalid register                                  */
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    As required by user code                                            */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/*                                                                        */ 
/**************************************************************************/ 
RETVAL regWrite(UINT16 RegNum, INT32 RegVal)
{
	//	REGISTER_PUT            regWrite;
	if (getCoreTable() != NULL)
	{
		if ((coreTable->regWrite != NULL) && (RegNum >=1))
		{
			RegNum--;	// make 0 based
			return((*coreTable->regWrite)(RegNum, RegVal));
		}
	}
	return(ERROR_NOT_DEFINED);
}

/**************************************************************************/
/*                                                                        */
/*  FUNCTION (public)                                                     */
/*                                                                        */
/*    RETVAL regVRead                                                      */
/*                                                                        */
/*  DESCRIPTION                                                           */
/*                                                                        */
/*    This function allows the caller to read any type of data within     */
/*    a variant register, converting from the last written, if different. */
/*                                                                        */
/*  PARAMETERS                                                            */
/*                                                                        */
/*    TASK *task - Pointer to current Quickstep task or NULL if none.     */
/*                 Local registers are stored within the TASK structure   */
/*    UINT16 RegNum - Register desired, 1 based.                          */
/*    void *RegVal - pointer to store result of 'type' to, if string      */
/*                   then an array of at least 256 characters is required */
/*    int type - Information on what 'RegVal' points to:                  */
/*               VARIANT_INTEGER, VARIANT_FLOAT, VARIANT_DOUBLE,          */
/*               VARIANT_STRING, VARIANT_STORAGE_TYPE, or                 */
/*               VARIANT_BASE_PROPERTIES_TYPE                             */
/*                                                                        */
/*               Note 1:  When using VARIANT_STORAGE_TYPE its internal    */
/*               settings effect the call:                                */
/*               ->type = Specifies the storage type desired where        */
/*                        VARIANT_CURRENT_TYPE means last written type.   */
/*                        Upon return this field is updated with type.    */
/*                        Specific types may also be requested (VARIANT_..*/
/*               ->indexX= -1 if use base Variant index pointer otherwise */
/*                        specifies the array element to write to.        */
/*               ->indexY= if not used leave at 0, else Y array dimension */
/*               ->precision= Number of digits of double/float precision  */
/*                        if reading that type.  Used for string conv.    */
/*                        when needed.  Default is 6.                     */
/*               Note 2:  When using VARIANT_BASE_PROPERTIES_TYPE it is   */
/*                        used to retrieve the current internal properties*/
/*                        of the variant based upon index/Y.              */
/*                                                                        */
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  RETURNS                                                               */
/*                                                                        */
/*    SUCCESS if conversion successful                                    */
/*    ERROR_NO_REG if invalid register                                    */
/*    ERROR_NOT_DEFINED there is no API access???                         */
/*                                                                        */
/**************************************************************************/
RETVAL regVRead( void *task, UINT16 RegNum, void *RegVal, int type)
{
	//	REGISTER_VGET            regVRead;
	if (getCoreTable() != NULL)
	{
		if ((coreTable->regVRead != NULL) && (RegNum >=1))
		{
			RegNum--;	// make 0 based
			return((*coreTable->regVRead)(task, RegNum, RegVal, type));
		}
	}
	return(ERROR_NOT_DEFINED);
}


/**************************************************************************/
/*                                                                        */
/*  FUNCTION (public)                                                     */
/*                                                                        */
/*    RETVAL regVWrite                                                     */
/*                                                                        */
/*  DESCRIPTION                                                           */
/*                                                                        */
/*    This function allows the caller to read any type of data within     */
/*    a variant register, converting from the last written, if different. */
/*                                                                        */
/*  PARAMETERS                                                            */
/*                                                                        */
/*    TASK *task - Pointer to current Quickstep task or NULL if none.     */
/*                 Local registers are stored within the TASK structure   */
/*    UINT16 RegNum - Register desired, 1's based                         */
/*    void *RegVal - pointer to get data to store of 'type' to, if string */
/*                   then an array of at least 256 characters is required */
/*    int type - Information on what 'RegVal' points to:                  */
/*               VARIANT_INTEGER, VARIANT_FLOAT, VARIANT_DOUBLE,          */
/*               VARIANT_STRING, or VARIANT_STORAGE_TYPE                  */
/*                                                                        */
/*               Note:  When using VARIANT_STORAGE_TYPE its internal      */
/*               settings effect the call:                                */
/*               ->type = Specifies the storage type within the structure */
/*               ->indexX= -1 if use base Variant index pointer otherwise */
/*                        specifies the array element to write to.        */
/*               ->indexY= if not used leave at 0, else Y array dimension */
/*               ->precision= Number of digits of double/float precision  */
/*                        if writing that type.  Used for string conv.    */
/*                        when needed.                                    */
/*                                                                        */
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  RETURNS                                                               */
/*                                                                        */
/*    SUCCESS if conversion successful                                    */
/*    ERROR_NO_REG if invalid register                                    */
/*    ERROR_NOT_DEFINED there is no API access???                         */
/*                                                                        */
/**************************************************************************/
RETVAL regVWrite( void *task, UINT16 RegNum, void *RegVal, int type)
{
	//	REGISTER_PUT            regWrite;
	if (getCoreTable() != NULL)
	{
		if ((coreTable->regVWrite != NULL) && (RegNum >=1))
		{
			RegNum--;	// make 0 based
			return((*coreTable->regVWrite)(task, RegNum, RegVal, type));
		}
	}
	return(ERROR_NOT_DEFINED);
}
/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    regLock                                             PORTABLE C      */ 
/*                                                           1.1          */ 
/*  DESCRIPTION                                                           */ 
/*    Request a lock on a register group by specifying a specific reg.    */
/*    Lower level mutex will be obtained, if not already owned, suggest   */
/*    unlocking in the reverse order locked.                              */
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    UINT16 RegNum - Register number from 1 to 64536 to read             */ 
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*    NONE                                                                */ 
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    As required by user code                                            */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/*                                                                        */ 
/**************************************************************************/ 
void regLock(UINT16 RegNum)
{
	if (getCoreTable() != NULL)
	{
		if ((coreTable->regLock != NULL) && (RegNum >=1))
		{
			RegNum--;	// make 0 based
			return((*coreTable->regLock)(RegNum));
		}
	}
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    regUnlock                                           PORTABLE C      */ 
/*                                                           1.1          */ 
/*  DESCRIPTION                                                           */ 
/*    Request an unlock on a register group by specifying a specific reg. */
/*    Lower level mutex will be obtained, if not already owned, suggest   */
/*    unlocking in the reverse order locked.                              */
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    UINT16 RegNum - Register number from 1 to 64536 to read             */ 
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*    NONE                                                                */ 
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    As required by user code                                            */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/*                                                                        */ 
/**************************************************************************/ 
void regUnlock(UINT16 RegNum)
{
	if (getCoreTable() != NULL)
	{
		if ((coreTable->regUnlock != NULL) && (RegNum >=1))
		{
			RegNum--;	// make 0 based
			(*coreTable->regUnlock)(RegNum);
		}
	}
}

/************************* Task Control   *********************************/
/*                                                                        */
/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    createTask                                          PORTABLE C      */ 
/*                                                           1.1          */ 
/*  DESCRIPTION                                                           */ 
/*    Request that the Quickstep task be created and started on the       */
/*    next Quickstep execution loop.                                      */
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    int stepNum - Step number to begin task execution on, 0 is first    */
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*    TASK * - NULL if no more tasks available, else pointer to TASK      */
/*             structure created, prior to execution.                     */
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    As required by user code                                            */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/*                                                                        */ 
/**************************************************************************/ 
TASK *createTask(int stepNum)
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->createTask != NULL)
		{
			return((*coreTable->createTask)(stepNum));
		}
	}
	return(NULL);
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    getTaskFromHandle                                   PORTABLE C      */ 
/*                                                           1.1          */ 
/*  DESCRIPTION                                                           */ 
/*    Each Task has a unique numeric handle that maps to a TASK *         */
/*    structure.  A call to this routine will return a that pointer or    */
/*    NULL if none is currently in existance.                             */
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    unsigned int handle - Unique task handle                            */ 
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*    TASK * - NULL if no task matches the passed handle, else TASK *     */
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    As required by user code                                            */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/*                                                                        */ 
/**************************************************************************/ 
TASK *getTaskFromHandle(unsigned int handle)
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->getTaskFromHandle != NULL)
		{
			return((*coreTable->getTaskFromHandle)(handle));
		}
	}
	return(NULL);
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    killTask                                            PORTABLE C      */ 
/*                                                           1.1          */ 
/*  DESCRIPTION                                                           */ 
/*    Request that the Quickstep task be created and started on the       */
/*    next Quickstep execution loop.                                      */
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    void *task - TASK * Quickstep task control block to kill.           */ 
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*    SUCCESS - 0                                                         */
/*    ERROR_TASK if task does not exist (null task pointer)               */
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    As required by user code                                            */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/*                                                                        */ 
/**************************************************************************/ 
RETVAL killTask(TASK *task)
{
	if (getCoreTable() != NULL)
	{
		if ((coreTable->killTask != NULL) && (task != NULL))
		{
			return((*coreTable->killTask)(task));
		}
	}
	return(ERROR_TASK);
}


/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    taskSetBranch                                       PORTABLE C      */ 
/*                                                           1.1          */ 
/*  DESCRIPTION                                                           */ 
/*    Request that the Quickstep task that was running when this 'C' API  */
/*    was called branch to 'step' upon immediate return from the 'C' API. */
/*    The branch will result in another task taking control prior to      */
/*    execution unless the 'taskStepLock' is invoked first.               */
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    void *task - TASK * passed to the resource filter when invoked.     */ 
/*    int step - Step number to branch to, 0 based where 0 is first.      */
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*    END_OF_STEP if branch is valid and successful                       */
/*    ERROR_NO_STEP if step does not exist                                */
/*    ERROR_TASK if task does not exist (null task pointer)               */
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    As required by user code                                            */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/*                                                                        */ 
/**************************************************************************/ 
RETVAL taskSetBranch( void *task, int step)
{
	if (getCoreTable() != NULL)
	{
		if ((coreTable->taskSetBranch != NULL) && (task != NULL))
		{
			return((*coreTable->taskSetBranch)(task,step));
		}
	}
	return(ERROR_TASK);
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    taskStepLock                                        PORTABLE C      */ 
/*                                                           1.1          */ 
/*  DESCRIPTION                                                           */ 
/*    Request that the Quickstep task that was running when this 'C' API  */
/*    was called run the next Quickstep step without switching control    */
/*    to another TASK, upon return.  Flag Lock cleared after next step    */
/*    is executed.                                                        */
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    void *task - TASK * passed to the resource filter when invoked.     */ 
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*    NONE                                                                */ 
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    As required by user code                                            */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/*                                                                        */ 
/**************************************************************************/ 
void taskStepLock(void *task)
{
	if (getCoreTable() != NULL)
	{
		if ((coreTable->taskStepLock != NULL) && (task != NULL))
		{
				// make 0 based
			(*coreTable->taskStepLock)(task);
		}
	}
}


/************************* Communications *********************************/
/*                                                                        */
/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    commSendMsg                                         PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*    Send a buffer of characters out a serial communications port        */
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    INDX port - Serial port to send buffer out on, 1 is COM1, 2 is COM2,*/ 
/*                virtual TCP connections are 3 to 7                      */
/*    UINT8 *msg - Pointer to unsigned character buffer containing message*/ 
/*    UINT16 size - Length of message to send                             */
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*    RETVAL -                                                            */ 
/*      SUCCESS = function called properly                                */
/*      ERROR_NO_COMM_PORT = Not a defined COMM port                      */ 
/*      ERROR_NOT_DEFINED  = Quickstep OS table not found                 */
/*      (also register specific return values defined in Errors.h)        */ 
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    As required by user code                                            */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/*                                                                        */ 
/**************************************************************************/ 
RETVAL commSendMsg( INDX port, UINT8 *msg, UINT16 size)
{
	//	COMM_SENDMSG            commSendMsg;
	if (getCoreTable() != NULL)
	{
		if (coreTable->commSendMsg != NULL)
		{
			return((*coreTable->commSendMsg)(port, msg, size));
		}
	}
	return(ERROR_NOT_DEFINED);
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    commGenericCmd                                      PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*     This function sends a generic message command (undefined format) of*/
/*     length "size" to the specified "port" using the appropriate driver.*/
/*     Any response is copied back to "msg"; the size of the of the       */
/*     response it copies back to "size".                                 */
/*     Messages can be used to change baud rate, etc...                   */
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    INDX port - Serial port to send buffer out on, 1 is COM1, 2 is COM2,*/ 
/*                virtual TCP connections are 3 to 7                      */
/*    UINT8 *msg - Pointer to unsigned character buffer containing message*/ 
/*    UINT16 *size - Pointer to unsigned short to store result length in  */
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*    RETVAL -                                                            */ 
/*      SUCCESS = function called properly                                */ 
/*      ERROR_NOT_DEFINED = Quickstep OS table not found                  */
/*      (also register specific return values defined in Errors.h)        */ 
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    As required by user code                                            */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/*                                                                        */ 
/**************************************************************************/ 
/*  Below are msg[] contents commands for each available.  The command must*/
/*  be the first byte of msg[] upon calling the function.
COMMCMD_RQST_QUERY - READ
	Check if the transmitter is free for message sending.
	Upon return:
	msg[0] = 0x00 if free.
	msg[1] = 0x01 if busy.
	*size = 1.
COMMCMD_RQST_CLRBUF - WRITE
	Clear the communications receive buffer.
	Upon return:
	msg[0] = COMMCMD_RSPN_ACK;
	*size = 1;
COMMCMD_RQST_PARSING - WRITE
	Turn parsing on or off based on the third byte of the message.
	Set msg to:
	msg[2] = 0 then disable parsing
	msg[2] = 1 then enable parsing
	Upon return:
	msg[0] = COMMCMD_RSPN_ACK;
	*size = 1;
COMMCMD_RQST_GETCNT - READ
	Return the current receive buffer count.
	Upon return:
	msg[0] = current count
	*size = 1;
COMMCMD_RQST_GETCH - READ
	Retrieve the nth character in the receive buffer.
	Set msg to:
	msg[2] = offset in buffer with 0 being first character
	Upon return:
	msg[0] = character at position requested
	*size = 1;
COMMCMD_RQST_SET_MODBUS - WRITE
	Activate/Deactivate Serial port modbus and set global port to use.
	Set msg to:
	msg[2] = Modbus RTU serial port address 1 to 254, also enables, 0 disables
	Upon return:
	msg[0] = COMMCMD_RSPN_ACK;
	*size = 1;
COMMCMD_RQST_NEWBAUD - WRITE
	Change the baud rate on the serial port (only physical, not virtual work)
	msg[2] = baud rate desired where 1 = 600, 2 = 1200, 3 = 2400, 4 = 4800,
	         5 = 9600, 6 = 19200 (default at powerup), 7 = 38400.
	Upon return:
	msg[0] = COMMCMD_RSPN_NACK if bad value, or COMMCMD_RSPN_ACK if OK
	*size = 1;
*/
RETVAL commGenericCmd( INDX port, UINT8 *msg, UINT16 *size)
{
	//	COMM_GENERICCMD         commGenericCmd;
	if (getCoreTable() != NULL)
	{
		if (coreTable->commGenericCmd != NULL)
		{
			return((*coreTable->commGenericCmd)(port, msg, size));
		}
	}
	return(ERROR_NOT_DEFINED);
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    commReadMsg                                         PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*     This function invokes the driver associated with "port" and        */
/*     returns a pointer to the buffer that contains the request message. */
/*     The message contents are NOT copied.  If a request message from    */
/*     the port is not available, the "buf" pointer is set to NULL.  The  */
/*     message contents are assumed to be encoded in one of the standard  */
/*     protocols.  Size is set to the number of received bytes in the     */
/*     buffer, not the number of bytes in the message.                    */
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    INDX port - Serial port to send buffer out on, 1 is COM1, 2 is COM2,*/ 
/*                virtual TCP connections are 3 to 7                      */
/*    UINT8 **buf - Pointer to unsigned character buffer stored here or   */ 
/*                  NULL if no message                                    */
/*    UINT16 *size - Number of bytes in buffer                            */ 
/*  OUTPUT                                                                */ 
/*    RETVAL -                                                            */ 
/*      SUCCESS = function called properly                                */ 
/*      ERROR_NO_COMM_PORT = Not a defined COMM port                      */ 
/*      ERROR_NOT_DEFINED  = Quickstep OS table not found                 */
/*      (also register specific return values defined in Errors.h)        */ 
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    As required by user code                                            */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/*                                                                        */ 
/**************************************************************************/ 
RETVAL commReadMsg( INDX port, UINT8 **buf,UINT16 *size)
{
	//	COMM_GET_RQST           commReadMsg;
	if (getCoreTable() != NULL)
	{
		if (coreTable->commReadMsg != NULL)
		{
			return((*coreTable->commReadMsg)(port, buf,size));
		}
	}
	return(ERROR_NOT_DEFINED);
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    commNetworkOpen                                     PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*     This function requests that a Network UDP or TCP port be openned.  */
/*     A RX thread will be spawned to automatically monitor the socket    */
/*     state.  This functions a USERCONNECTION control block that must    */
/*     not be modfied after a successful call and should be treated as    */
/*     a connection HANDLE on other Network calls.  A callback routine    */
/*     must be suppled as one of the USERCONNECTION parameters which will */
/*     be invoked whenever a change of state occurs, including packet     */
/*     reception.                                                         */
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    USERCONNECTION *user - Structure used to pass  connection/socket    */
/*                information, must be initialized prior to call.  Only   */
/*                network type NETWORK_TYPE_UDP currently supported       */
/*  OUTPUT                                                                */ 
/*    int - 0 if success and -1 if failed                                 */ 
/*      user->state set to USER_FAILED or USER_CONNECTING                 */ 
/*      NOTE:  Once user->state becomes USER_CONNECTED this structure     */
/*             may be modifed and used for commNetworkSend operations     */
/*             It is cloned by the network thread.  The copy is what is   */
/*             modified and supplied to the callback function.            */
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    As required by user code                                            */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/*                                                                        */ 
/**************************************************************************/ 
int commNetworkOpen( USERCONNECTION *user)
{
	//	COMM_NETWORKOPEN    commNetworkOpen;
	if (getCoreTable() != NULL)
	{
		if (coreTable->commNetworkOpen != NULL)
		{
			return((*coreTable->commNetworkOpen)(user));
		}
	}
	return(-1);
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    commNetworkClose                                    PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*     This function requests that an existing network connection/socket  */
/*     created with a call to commNetworkOpen, be closed.  The same       */
/*     USERCONNECTION structure passed to the commNetworkOpen call should */
/*     be passed to this function.                                        */
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    USERCONNECTION *user - Structure used to pass  connection/socket    */
/*                information, must be initialized prior to call.  Only   */
/*                network type NETWORK_TYPE_UDP currently supported       */
/*                Should be same USERCONNECTION passed to commNetworkOpen */
/*                                                                        */
/*  OUTPUT                                                                */ 
/*    NONE                                                                */ 
/*    user->state modified to either USER_FAILED or USER_DISCONNECTED     */
/*                                                                        */
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    As required by user code                                            */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/*                                                                        */ 
/**************************************************************************/ 
void commNetworkClose( USERCONNECTION *user)
{
	//	COMM_NETWORKCLOSE    commNetworkClose;
	if (getCoreTable() != NULL)
	{
		if (coreTable->commNetworkClose != NULL)
		{
			(*coreTable->commNetworkClose)(user);
			return;
		}
	}
	return;
}
/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    commNetworkSend                                     PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*     This function requests that a packet be sent on a connection       */
/*     openned by commNetworkOpen.  The USERCONNECTION control block      */
/*     must contain the proper information.                               */
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    USERCONNECTION *user - Structure returned by commNetworkOpen with   */
/*          following parameters set as desired:                          */
/*            packetBuf - pointer to buffer to send                       */
/*            length - number of bytes to send                            */
/*            destIp - If UDP, destination IP address                     */
/*            destPort - if UDP, destination port address                 */
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*    int - number of bytes queued                                        */ 
/*    user->state set to USER_FAILED or USER_DATAQUEUED                   */
/*      NOTE:  Limited queuing is available at the network stack level    */ 
/*             exceeding this will result in packet loss.  Do not send    */
/*             a large number of packets to the host without using a      */
/*             protocol that can verify packets have been transfered.     */
/*             This is typically only a problem if you attempt to sit in a*/
/*             loop continually transmitting data.                        */
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    As required by user code                                            */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/*                                                                        */ 
/**************************************************************************/ 
int commNetworkSend( USERCONNECTION *user)
{
	//	COMM_NETWORKSEND    commNetworkSend;
	if (getCoreTable() != NULL)
	{
		if (coreTable->commNetworkSend != NULL)
		{
			return((*coreTable->commNetworkSend)(user));
		}
	}
	return(-1);
}

/************************* System Routines ********************************/
/*                                                                        */
/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    systemTics                                          PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*    This function is called to get the number of milliseconds that have */
/*    occured since powerup.  Note that this will wrap back to 0.         */ 
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    none                                                                */ 
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*    unsigned long - number of system tics, in milliseconds since powerup*/ 
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    As required by user code                                            */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/*                                                                        */ 
/**************************************************************************/ 
unsigned long systemTics(void)		// number of timer tics since powerup, 1ms/tic currently
	{
	//	GET_SYSTEM_TICS         systemTics;
	if (getCoreTable() != NULL)
	{
		if (coreTable->systemTics != NULL)
		{
			return((*coreTable->systemTics)());
		}
	}
	return(0);
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    fireWatchdog                                        PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*    This function is called to reset the watchdog system timer that     */
/*    will cause a system reset and fault if not invoked every 86 ms.     */ 
/*    Quickstep OS will automatically call this function as required      */ 
/*    but if a User Function runs as a step it must make the call if it   */ 
/*    maintains control too long, preventing Quickstep OS from calling    */
/*    the function.                                                       */
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    none                                                                */ 
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*    RETVAL -                                                            */ 
/*      SUCCESS = function called properly                                */ 
/*      ERROR_NOT_DEFINED = Quickstep OS table not found                  */ 
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    As required by user code                                            */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/*                                                                        */ 
/**************************************************************************/ 
RETVAL fireWatchdog(void)		
	{
	//	FIREWATCHDOG         fireWatchdog;
	if (getCoreTable() != NULL)
	{
		if (coreTable->fireWatchdog != NULL)
		{
			(*coreTable->fireWatchdog)();
			return (SUCCESS);
		}
	}
	return(ERROR_NOT_DEFINED);
}

/***************************** Step Control *******************************/
/*                                                                        */
RETVAL enterStepAtomicity(void)
{
	//	ENTER_STEP_ATOMICITY    enterStepAtomicity;
	if (getCoreTable() != NULL)
	{
		if (coreTable->enterStepAtomicity != NULL)
		{
			return((*coreTable->enterStepAtomicity)());
		}
	}
	return(ERROR_NOT_DEFINED);
}
RETVAL exitStepAtomicity(void)
{
	//	EXIT_STEP_ATOMICITY     exitStepAtomicity;
	if (getCoreTable() != NULL)
	{
		if (coreTable->exitStepAtomicity != NULL)
		{
			return((*coreTable->exitStepAtomicity)());
		}
	}
	return(ERROR_NOT_DEFINED);
}

/************************* Resource Filter Routines ***********************/
/*                                                                        */
/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    addResourceFilter                                   PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*    This function is called to add a resource filter to the Quickstep   */
/*    list.  The resource to monitor is defined in the RESOURCE_INFO      */
/*    structure and passed as a parameter.  A pointer to the "C" function */
/*    to call upon access is also passed                                  */ 
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    RESOURCE_INFO *rsc - Pointer to structure defining resource and     */
/*       access method upon which to invoke the passed function           */ 
/*    FILTER_FUNCTION func - Pointer to "C" function to call when the     */
/*    parameters of the RESOURCE_INFO structure are satisfied             */
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*    void *handle - Handle returned by addResourceFilter or NULL if 	  */
/*                   failed                                               */ 
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    As required by user code                                            */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/*                                                                        */ 
/**************************************************************************/ 
/****************************************************************
*  RESOURCE_INFO rsc - addResourceFilter parameter block        *
*																*
*  This parameter block must be filled out prior to registering *
*  a callback function for a Quickstep resource.  A pointer to  *
*  it is passed to the addResourceFilter() function             *
****************************************************************/
/*
typedef struct resourceInfo {
	int type;		// Type of resource to add filter to, RESOURCE_ANALOGIN, RESOURCE_ANALOGOUT, etc...
	int mode;		// Type of access to invoke filter on when accessed, RESOURCE_READ, RESOURCE_WRITE
	int start;		// Start number of resource (setting a range)
	int end;		// End number of resource, make same as start if only one (setting a range).
} RESOURCE_INFO;
*/
/****************************************************************
* FILTER_FUNCTION func                      			        *
*																*
* Type definition for Resource Callback function			    *
*																*
* PARAMETERS:													*
* void *handle - handle returned by addUserResourceFilter when  *
*                function was registered                        *
* FILTERPARAMS *params - Access information block to detail     *
*                        what is being done                     *
* STDVAL value - Current value being read or written.           *
* RETVAL *status - pointer to status code that will be returned *
*                  to Quickstep.  Leaving it unchanged will     *
*                  default to SUCCESS.  Use only for defined    *
*                  errors.                                      *
*																*
* RETURNS:														*
* int - new value to return to Quickstep on a read or value to  *
*       write, if no change then return passed "value"          *
*****************************************************************/
void *addResourceFilter(RESOURCE_INFO *rsc, FILTER_FUNCTION func)	// add a resource filter function
	{
	//	ADD_RESOURCE_FILTER     addResourceFilter;
	if (getCoreTable() != NULL)
	{
		if (coreTable->addResourceFilter != NULL)
		{
			return((*coreTable->addResourceFilter)(rsc,func));
		}
	}
	return(NULL);
}
/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    removeResourceFilter                                PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*    This function is called to remove a resource filter from the        */
/*    Quickstep list.  A filter is removed by passing the handle that     */ 
/*    was returned when it was first added.                               */
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    void *handle - Handle returned by addResourceFilter                 */ 
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*    RETVAL -                                                            */ 
/*      SUCCESS = function called properly                                */ 
/*      ERROR_NOT_DEFINED = Quickstep OS table not found                  */ 
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    As required by user code                                            */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/*                                                                        */ 
/**************************************************************************/ 
RETVAL removeResourceFilter(void *handle)		// remove a resource filter function
	{
	//	REMOVE_RESOURCE_FILTER  removeResourceFilter;
	if (getCoreTable() != NULL)
	{
		if (coreTable->removeResourceFilter != NULL)
		{
			return((*coreTable->removeResourceFilter)(handle));
		}
	}
	return(ERROR_NOT_DEFINED);
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    CTC_allocateCommandParser                           PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*    This function is used to allocate an environment for running        */
/*    script commands.  It may be used for multiple commands.  Since      */ 
/*    a malloc call is involved it is best to reserve a pool than call    */
/*    and release after every use.                                        */ 
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    int maxbuffer - Maximum amount of message response dumped to the    */
/*    telnet screen to copy to an internal buffer.  For example if you    */
/*    wanted an entire directory this might have to be 2048 whereas       */
/*    simple commands may only be 256.  Typically 512 is adequate and     */
/*    regardless of output will not exceed this value (buffer truncated). */
/*    A value of 0 will allow commands to run but no result string is     */
/*    buffered.                                                           */
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*    RETVAL -                                                            */ 
/*      CMDPARSER * = Environment handle/pointer for additional calls to  */
/*               other Script command functions.  Actually CMDPARSER      */
/*               structure pointer should you attempt to access directly  */
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    As required by user code                                            */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/*                                                                        */ 
/**************************************************************************/ 
CMDPARSER *CTC_allocateCommandParser(int maxbuffer)
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->allocateCommandParser != NULL)
		{
			return((*coreTable->allocateCommandParser)(maxbuffer));
		}
	}
	return NULL;
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    CTC_releaseCommandParser                            PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*    This function is used to return the allocated Script Command        */
/*    environment, freeing memory usage.                                  */ 
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    CMDPARSER *pCmdP - Pointer returned by the CTC_allocateCommandPaser */
/*    function call.                                                      */
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*    None                                                                */ 
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    As required by user code                                            */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/*                                                                        */ 
/**************************************************************************/ 
void CTC_releaseCommandParser(CMDPARSER *pCmdP)
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->releaseCommandParser != NULL)
		{
			(*coreTable->releaseCommandParser)(pCmdP);
		}
	}
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    CTC_runCommandParser                                PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*    This function is used to execute any desired script command.        */
/*    All responses returned to the telnet screen will be written         */ 
/*    directly to a memory buffer for later analysis by the caller,       */
/*    This includes all CRLF combinations, etc.  Screen output is simply  */ 
/*    redirected to a buffer instead of a telnet or serial diagnostic     */ 
/*    screen.  Most commands begin with ERROR or SUCCESS with a possible  */
/*    leading white space or CRLF combination.                            */                                          
/*  INPUT                                                                 */ 
/*    CMDPARSER *pCmdP - Pointer returned by the CTC_allocateCommandPaser */
/*    function call.                                                      */
/*                                                                        */
/*    char *cmd - Null terminated string of command to execute.           */
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*    long - 0, if no error, else Script Error Mask (defined in           */ 
/*           CoreFunc.c                                                   */
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    As required by user code                                            */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/*                                                                        */ 
/**************************************************************************/ 
long CTC_runCommandParser(CMDPARSER *pCmdP, char *cmd)
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->runCommandParser != NULL)
		{
			return((*coreTable->runCommandParser)(pCmdP, cmd));
		}
	}
	return SCRIPT_ERRMASK_FATAL;
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    CTC_getCommandResponse                              PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*    This function is used to copy the command result response string    */
/*    to another buffer, length of that copied is returned.  Alternatively*/ 
/*    you may access the CMDPARSER structure itself.                      */
/*                                                                        */
/*  INPUT                                                                 */ 
/*    CMDPARSER *pCmdP - Pointer returned by the CTC_allocateCommandPaser */
/*    function call.                                                      */
/*                                                                        */
/*    char *buf - User buffer to copy result message to, null terminated. */
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*    int - Number of characters copied to the user buffer, buf.          */ 
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    As required by user code                                            */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/*                                                                        */ 
/**************************************************************************/ 
int CTC_getCommandResponse(CMDPARSER *pCmdP,char *buf)
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->getCommandResponse != NULL)
		{
			return((*coreTable->getCommandResponse)(pCmdP, buf));
		}
	}
	return 0;
}

/*************************** Advanced Threading ***************************/
/*                                                                        */
/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    _tx_thread_create                                   PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function creates a thread and places it on the list of created */ 
/*    threads.                                                            */ 
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*                                                                        */ 
/*    thread_ptr                            Thread control block pointer  */ 
/*    name                                  Pointer to thread name string */ 
/*    entry_function                        Entry function of the thread  */ 
/*    entry_input                           32-bit input value to thread  */ 
/*    stack_start                           Pointer to start of stack     */ 
/*    stack_size                            Stack size in bytes           */ 
/*    priority                              Priority of thread (0-31)     */ 
/*    preempt_threshold                     Preemption threshold          */
/*    time_slice                            Thread time-slice value       */ 
/*    auto_start                            Automatic start selection     */ 
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*                                                                        */ 
/*    return status                         Thread create return status   */ 
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    Application Code                                                    */ 
/*    _tx_timer_initialize                  Create system timer thread    */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/**************************************************************************/ 
unsigned int _tx_thread_create( TX_THREAD *thread_ptr,       /* Task Structure */
	/* Task Name */
	char *name_ptr,
	/* Routine and Parameter to pass */
	void (*entry_function)(unsigned long), unsigned long entry_input,
	                        /* Stack start and length */
	void *stack_start, unsigned long stack_size,
	                        /* Priority and Threshold */
	unsigned int priority, unsigned int preempt_threshold, 
	/* time slice */
	unsigned long time_slice, unsigned int auto_start)
{
	//	THREAD_CREATE           thread_create;
	if (getCoreTable() != NULL)
	{
		if (coreTable->thread_create != NULL)
		{
			return((*coreTable->thread_create)(thread_ptr,name_ptr,entry_function,
				entry_input,stack_start,stack_size,priority,preempt_threshold,time_slice,auto_start));
		}
	}
	return(TX_THREAD_ERROR);
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    _tx_thread_suspend                                  PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function handles application suspend requests.  If the suspend */ 
/*    requires actual processing, this function calls the actual suspend  */ 
/*    thread routine.                                                     */ 
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*                                                                        */ 
/*    thread_ptr                            Pointer to thread to suspend  */ 
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*                                                                        */ 
/*    status                                Return completion status      */ 
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    Application code                                                    */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/**************************************************************************/ 
unsigned int _tx_thread_suspend(TX_THREAD *thread_ptr)  // suspend a thread
{
	//	THREAD_SUSPEND           thread_suspend;
	if (getCoreTable() != NULL)
	{
		if (coreTable->thread_suspend != NULL)
		{
			return((*coreTable->thread_suspend)(thread_ptr));
		}
	}
	return(TX_THREAD_ERROR);
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    _tx_thread_resume                                   PORTABLE C      */ 
/*                                                                        */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function processes application resume thread services.  Actual */ 
/*    thread resumption is performed in the core service.                 */ 
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*                                                                        */ 
/*    thread_ptr                            Pointer to thread to resume   */ 
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*                                                                        */ 
/*    status                                Service return status         */ 
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    Application Code                                                    */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/**************************************************************************/ 
unsigned int _tx_thread_resume(TX_THREAD *thread_ptr)   // resume a suspended thread
{
	//	THREAD_RESUME           thread_resume;
	if (getCoreTable() != NULL)
	{
		if (coreTable->thread_resume != NULL)
		{
			return((*coreTable->thread_resume)(thread_ptr));
		}
	}
	return(TX_THREAD_ERROR);
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    _tx_thread_sleep                                    PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function handles application thread sleep requests.  If the    */ 
/*    sleep request was called from a non-thread, an error is returned.   */ 
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*                                                                        */ 
/*    timer_ticks                           Number of timer ticks to sleep*/ 
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*                                                                        */ 
/*    status                                Return completion status      */ 
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    _tx_timer_activate                    Activate sleep timer          */ 
/*    _tx_thread_suspend                    Actual thread suspension      */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*                                                                        */ 
/*    Application code                                                    */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/**************************************************************************/ 
unsigned int _tx_thread_sleep(unsigned long tics)	// amount of time to sleep, only call from a thread
	{
	//	THREAD_SLEEP           thread_sleep;
	if (getCoreTable() != NULL)
	{
		if (coreTable->thread_sleep != NULL)
		{
			return((*coreTable->thread_sleep)(tics));
		}
	}
	return(TX_THREAD_ERROR);
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    _tx_mutex_create                                    PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function creates a mutex with optional priority inheritance as */ 
/*    specified in this call.                                             */ 
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    mutex_ptr                             Pointer to mutex control block*/ 
/*    name_ptr                              Pointer to mutex name         */ 
/*    inherit                               Priority inheritance option   */ 
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*    TX_SUCCESS                        Successful completion status      */ 
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*                                                                        */ 
/*    None                                                                */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    Application Code                                                    */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/**************************************************************************/ 
unsigned int _txe_mutex_create(TX_MUTEX *mutex_ptr, char *name, unsigned int inherit)
{
	//	MUTEX_CREATE            mutex_create;
	if (getCoreTable() != NULL)
	{
		if (coreTable->mutex_create != NULL)
		{
			return((*coreTable->mutex_create)(mutex_ptr, name, inherit));
		}
	}
	return(TX_MUTEX_ERROR);
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    _tx_mutex_get                                       PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function gets the specified mutex.  If the calling thread      */ 
/*    already owns the mutex, an ownership count is simply increased.     */ 
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    mutex_ptr                         Pointer to mutex control block    */ 
/*    wait_option                       Suspension option                 */ 
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*    status                            Completion status                 */ 
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    _tx_timer_activate                Activate timer routine            */ 
/*    _tx_thread_suspend                Suspend thread service            */ 
/*    _tx_mutex_priority_change         Inherit thread priority           */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    Application Code                                                    */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/**************************************************************************/ 
unsigned int _txe_mutex_get(TX_MUTEX *mutex_ptr, unsigned long wait_option)
{
	//	MUTEX_GET               mutex_get;
	if (getCoreTable() != NULL)
	{
		if (coreTable->mutex_get != NULL)
		{
			return((*coreTable->mutex_get)(mutex_ptr, wait_option));
		}
	}
	return(TX_MUTEX_ERROR);
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    _tx_mutex_put                                       PORTABLE C      */ 
/*                                                           1.0          */  
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function puts back an instance of the specified mutex.         */ 
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    mutex_ptr                         Pointer to mutex control block    */ 
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*    TX_SUCCESS                        Success completion status         */ 
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    _tx_timer_deactivate              Deactivate timer routine          */ 
/*    _tx_thread_resume                 Resume thread service             */ 
/*    _tx_thread_system_return          Return to system routine          */ 
/*    _tx_mutex_priority_change         Restore previous thread priority  */ 
/*    _tx_mutex_prioritize              Prioritize the mutex suspension   */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    Application Code                                                    */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/**************************************************************************/ 
unsigned int _txe_mutex_put(TX_MUTEX *mutex_ptr)
{
	//	MUTEX_PUT               mutex_put;
	if (getCoreTable() != NULL)
	{
		if (coreTable->mutex_put != NULL)
		{
			return((*coreTable->mutex_put)(mutex_ptr));
		}
	}
	return(TX_MUTEX_ERROR);
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION                                               RELEASE        */ 
/*                                                                        */ 
/*    _tx_mutex_delete                                    PORTABLE C      */ 
/*                                                           1.0          */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function deletes the specified mutex.  All threads             */ 
/*    suspended on the mutex are resumed with the TX_DELETED status       */ 
/*    code.                                                               */ 
/*                                                                        */ 
/*  INPUT                                                                 */ 
/*    mutex_ptr                         Pointer to mutex control block    */ 
/*                                                                        */ 
/*  OUTPUT                                                                */ 
/*    TX_SUCCESS                        Successful completion status      */ 
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    _tx_timer_deactivate              Deactivate timer routine          */ 
/*    _tx_thread_resume                 Resume thread service             */ 
/*                                                                        */ 
/*  CALLED BY                                                             */ 
/*    Application Code                                                    */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                      DESCRIPTION             */ 
/*                                                                        */ 
/**************************************************************************/ 
unsigned int _txe_mutex_delete(TX_MUTEX *mutex_ptr)
{
	//	MUTEX_PUT               mutex_put;
	if (getCoreTable() != NULL)
	{
		if (coreTable->mutex_delete != NULL)
		{
			return((*coreTable->mutex_delete)(mutex_ptr));
		}
	}
	return(TX_MUTEX_ERROR);
}

/**************************** Diagnostics *********************************/
/*                                                                        */
RETVAL logEvent(EventCode event, long parameter)
{
	//	LOGEVENT                logEvent;
	if (getCoreTable() != NULL)
	{
		if (coreTable->logEvent != NULL)
		{
			return((*coreTable->logEvent)(event, parameter));
		}
	}
	return(ERROR_NOT_DEFINED);
}

RETVAL logEventPlus(EventCode event, long firstParameter, long secondParameter)
{
	//	LOGEVENTPLUS            logEventPlus;
	if (getCoreTable() != NULL)
	{
		if (coreTable->logEventPlus != NULL)
		{
			return((*coreTable->logEventPlus)(event, firstParameter, secondParameter));
		}
	}
	return(ERROR_NOT_DEFINED);
}

/******************** Reserved/not for user access ************************/
/*                                                                        */
unsigned int _tx_byte_allocate(TX_BYTE_POOL *pool_ptr, void **memory_ptr,
	unsigned long memory_size, unsigned long wait_option)
{
	//	BYTE_ALLOCATE           tx_byte_allocate;
	if (getCoreTable() != NULL)
	{
		if (coreTable->tx_byte_allocate != NULL)
		{
			return((*coreTable->tx_byte_allocate)(pool_ptr, memory_ptr, memory_size, wait_option));
		}
	}
	return(TX_POOL_ERROR);
}

unsigned int _tx_byte_release(void *memory_ptr)
{
	//	BYTE_RELEASE            tx_byte_release;
	if (getCoreTable() != NULL)
	{
		if (coreTable->tx_byte_release != NULL)
		{
			return((*coreTable->tx_byte_release)(memory_ptr));
		}
	}
	return(TX_POOL_ERROR);
}

unsigned int _tx_byte_pool_create(TX_BYTE_POOL *pool_ptr, char *name_ptr, 
	void *pool_start, unsigned long pool_size)
{
	//	BYTE_POOL_CREATE        tx_byte_pool_create;
	if (getCoreTable() != NULL)
	{
		if (coreTable->tx_byte_pool_create != NULL)
		{
			return((*coreTable->tx_byte_pool_create)(pool_ptr, name_ptr, pool_start, pool_size));
		}
	}
	return(TX_POOL_ERROR);
}

unsigned int _txe_byte_pool_delete(TX_BYTE_POOL *pool_ptr)
{
	//	BYTE_POOL_CREATE        tx_byte_pool_create;
	if (getCoreTable() != NULL)
	{
		if (coreTable->tx_byte_pool_delete != NULL)
		{
			return((*coreTable->tx_byte_pool_delete)(pool_ptr));
		}
	}
	return(TX_POOL_ERROR);
}
unsigned int _tx_byte_pool_delete(TX_BYTE_POOL *pool_ptr)
{
	//	BYTE_POOL_CREATE        tx_byte_pool_create;
	if (getCoreTable() != NULL)
	{
		if (coreTable->tx_byte_pool_delete != NULL)
		{
			return((*coreTable->tx_byte_pool_delete)(pool_ptr));
		}
	}
	return(TX_POOL_ERROR);
}

// interrupt control
unsigned int _tx_thread_interrupt_control(unsigned int state)
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->interruptControl != NULL)
		{
			return((*coreTable->interruptControl)(state));
		}
	}
	return(0);
}

/*************************************** FILE SYSTEM FUNCTIONS ***************************************/

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION (public)                                                     */ 
/*                                                                        */ 
/*    void CTC_fopen                                                      */ 
/*                                                                        */ 
/*  AUTHOR                                                                */ 
/*                                                                        */ 
/*    Kevin Halloran                                                      */ 
/*                                                                        */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function is called to open a file for access, similiar to      */ 
/*    UNIX and Microsoft fopen() calls.                                   */ 
/*                                                                        */ 
/*  PARAMETERS                                                            */ 
/*                                                                        */ 
/*    char *filename              Pointer to name of file to open, if     */ 
/*                                path information is not provided then   */
/*                                the current directory of the thread is  */
/*                                assumed.                                */
/*    char *mode                  Manner in which to open the file        */
/*                                   "w"  - write only, must exist        */
/*                                   "r"  - read only, must exist         */
/*                                   "w+" - read/write and create new     */
/*                                   "r+" - read/write, must exist        */
/*                                   "wb" - write only, must exist        */
/*                                   "wb+"- read/write, create new        */
/*                                   "rb+"- read/write, must exist        */
/*                                                                        */ 
/*                                   (all files openned in binary mode)   */
/*                                                                        */
/*  RETURNS                                                               */ 
/*                                                                        */ 
/*    CTC_FILE - File handle for read/write access                        */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                DESCRIPTION                   */ 
/*                                                                        */ 
/*  07-01-2003         Kevin H              Initial Version               */ 
/*                                                                        */ 
/**************************************************************************/ 
CTC_FILE CTC_fopen(char *filename, char *mode)
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->CTC_fopen != NULL)
		{
			return((*coreTable->CTC_fopen)(filename, mode));
		}
	}
	return NULL;
} 

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION (public)                                                     */ 
/*                                                                        */ 
/*    unsigned long CTC_filelength                                        */ 
/*                                                                        */ 
/*  AUTHOR                                                                */ 
/*                                                                        */ 
/*    Kevin Halloran                                                      */ 
/*                                                                        */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function is used to get the file size of an open, existing file*/ 
/*                                                                        */ 
/*  PARAMETERS                                                            */ 
/*                                                                        */ 
/*    CTC_FILE apFile                File handle returned by CTC_fopen()  */ 
/*                                                                        */ 
/*  RETURNS                                                               */ 
/*                                                                        */ 
/*    unsigned long - file size or 0 if none                              */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                DESCRIPTION                   */ 
/*                                                                        */ 
/*  07-01-2003         Kevin H              Initial Version               */ 
/*                                                                        */ 
/**************************************************************************/ 
unsigned long CTC_filelength(CTC_FILE apFile)
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->CTC_filelength != NULL)
		{
			return((*coreTable->CTC_filelength)(apFile));
		}
	}
	return 0;
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION (public)                                                     */ 
/*                                                                        */ 
/*    int CTC_fclose                                                      */ 
/*                                                                        */ 
/*  AUTHOR                                                                */ 
/*                                                                        */ 
/*    Kevin Halloran                                                      */ 
/*                                                                        */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function is used to close a file openned by a call to          */ 
/*    CTC_fopen()                                                         */
/*                                                                        */ 
/*  PARAMETERS                                                            */ 
/*                                                                        */ 
/*    CTC_FILE apFile                File handle returned by CTC_fopen()  */ 
/*                                                                        */ 
/*  RETURNS                                                               */ 
/*                                                                        */ 
/*    int - 0 if OK, -1 if failed                                         */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                DESCRIPTION                   */ 
/*                                                                        */ 
/*  07-01-2003         Kevin H              Initial Version               */ 
/*                                                                        */ 
/**************************************************************************/ 
int CTC_fclose(CTC_FILE apFile)
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->CTC_fclose != NULL)
		{
			return((*coreTable->CTC_fclose)(apFile));
		}
	}
	return -1;
} 

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION (public)                                                     */ 
/*                                                                        */ 
/*    int CTC_feof                                                        */ 
/*                                                                        */ 
/*  AUTHOR                                                                */ 
/*                                                                        */ 
/*    Kevin Halloran                                                      */ 
/*                                                                        */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function is used to detect the 'end of file' following         */ 
/*    CTC_fread or CTC_fgets calls                                        */
/*                                                                        */ 
/*  PARAMETERS                                                            */ 
/*                                                                        */ 
/*    CTC_FILE apFile                File handle returned by CTC_fopen()  */ 
/*                                                                        */ 
/*  RETURNS                                                               */ 
/*                                                                        */ 
/*    int - 0 if OK, -1 if end of file                                   */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                DESCRIPTION                   */ 
/*                                                                        */ 
/*  07-01-2003         Kevin H              Initial Version               */ 
/*                                                                        */ 
/**************************************************************************/ 
int CTC_feof( CTC_FILE apFile )
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->CTC_feof != NULL)
		{
			return((*coreTable->CTC_feof)(apFile));
		}
	}
	return -1;
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION (public)                                                     */ 
/*                                                                        */ 
/*    unsigned int CTC_fwrite                                             */ 
/*                                                                        */ 
/*  AUTHOR                                                                */ 
/*                                                                        */ 
/*    Kevin Halloran                                                      */ 
/*                                                                        */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function is used to write to a file openned by a call to       */ 
/*    CTC_fopen()                                                         */
/*                                                                        */ 
/*  PARAMETERS                                                            */ 
/*                                                                        */ 
/*    void *buffer                   Data to write of 'size' items        */ 
/*    unsigned int size              Size (width) of each item to write   */ 
/*    unsigned int count             Number of items to write             */ 
/*    CTC_FILE apFile                File handle returned by CTC_fopen()  */ 
/*                                                                        */ 
/*  RETURNS                                                               */ 
/*                                                                        */ 
/*    unsigned int - 0 if failed, else number of items written           */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                DESCRIPTION                   */ 
/*                                                                        */ 
/*  07-01-2003         Kevin H              Initial Version               */ 
/*                                                                        */ 
/**************************************************************************/ 
unsigned int CTC_fwrite( void *buffer, unsigned int size, unsigned int count, CTC_FILE apFile )
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->CTC_fwrite != NULL)
		{
			return((*coreTable->CTC_fwrite)(buffer,size, count, apFile ));
		}
	}
	return 0;
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION (public)                                                     */ 
/*                                                                        */ 
/*    int CTC_fputs                                                       */ 
/*                                                                        */ 
/*  AUTHOR                                                                */ 
/*                                                                        */ 
/*    Kevin Halloran                                                      */ 
/*                                                                        */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function is used to write a string to a file (no CRLF is       */
/*    is appended per standard).                                          */ 
/*                                                                        */ 
/*  PARAMETERS                                                            */ 
/*                                                                        */ 
/*    char *string                   Pointer on null terminated string to */
/*                                   write to file.                       */ 
/*    CTC_FILE apFile                File handle returned by CTC_fopen()  */ 
/*                                                                        */ 
/*  RETURNS                                                               */ 
/*                                                                        */ 
/*    0  - executed correctly                                             */ 
/*    -1 - failed                                                         */
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                DESCRIPTION                   */ 
/*                                                                        */ 
/*  07-01-2003         Kevin H              Initial Version               */ 
/*                                                                        */ 
/**************************************************************************/ 
int CTC_fputs( char *buffer, CTC_FILE apFile)
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->CTC_fputs != NULL)
		{
			return((*coreTable->CTC_fputs)( buffer, apFile));
		}
	}
	return -1;
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION (public)                                                     */ 
/*                                                                        */ 
/*    void CTC_fflush                                                     */ 
/*                                                                        */ 
/*  AUTHOR                                                                */ 
/*                                                                        */ 
/*    Kevin Halloran                                                      */ 
/*                                                                        */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function is used to flush write data to an open file. Only     */ 
/*    currently used for SDISK operation if leave open with write access. */
/*                                                                        */ 
/*  PARAMETERS                                                            */ 
/*                                                                        */ 
/*    CTC_FILE apFile                File handle returned by CTC_fopen()  */ 
/*                                                                        */ 
/*  RETURNS                                                               */ 
/*                                                                        */ 
/*    Nothing                                                             */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                DESCRIPTION                   */ 
/*                                                                        */ 
/*  07-01-2003         Kevin H              Initial Version               */ 
/*                                                                        */ 
/**************************************************************************/ 
void CTC_fflush(CTC_FILE apFile)
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->CTC_fflush != NULL)
		{
			((*coreTable->CTC_fflush)(apFile));
		}
	}
	return;
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION (public)                                                     */ 
/*                                                                        */ 
/*    long CTC_ftell                                                      */ 
/*                                                                        */ 
/*  AUTHOR                                                                */ 
/*                                                                        */ 
/*    Kevin Halloran                                                      */ 
/*                                                                        */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function is used to determine the present file position on     */ 
/*    a file openned by CTC_fopen()                                       */
/*                                                                        */ 
/*  PARAMETERS                                                            */ 
/*                                                                        */ 
/*    CTC_FILE apFile                File handle returned by CTC_fopen()  */ 
/*                                                                        */ 
/*  RETURNS                                                               */ 
/*                                                                        */ 
/*    long - -1 if failed, else current file position                     */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                DESCRIPTION                   */ 
/*                                                                        */ 
/*  07-01-2003         Kevin H              Initial Version               */ 
/*                                                                        */ 
/**************************************************************************/ 
long CTC_ftell(CTC_FILE apFile)
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->CTC_ftell != NULL)
		{
			return((*coreTable->CTC_ftell)( apFile));
		}
	}
	return -1;
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION (public)                                                     */ 
/*                                                                        */ 
/*    unsigned int CTC_fread                                              */ 
/*                                                                        */ 
/*  AUTHOR                                                                */ 
/*                                                                        */ 
/*    Kevin Halloran                                                      */ 
/*                                                                        */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function is used to read from a file openned by a call to      */ 
/*    CTC_fopen()                                                         */
/*                                                                        */ 
/*  PARAMETERS                                                            */ 
/*                                                                        */ 
/*    void *buffer                   Pointer on where to place read data  */ 
/*    unsigned int size              Size (width) of each item to read    */ 
/*    unsigned int count             Number of items to read              */ 
/*    CTC_FILE apFile                File handle returned by CTC_fopen()  */ 
/*                                                                        */ 
/*  RETURNS                                                               */ 
/*                                                                        */ 
/*    unsigned int - 0 if failed, else number of items read              */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                DESCRIPTION                   */ 
/*                                                                        */ 
/*  07-01-2003         Kevin H              Initial Version               */ 
/*                                                                        */ 
/**************************************************************************/ 
unsigned int CTC_fread(void *apByteBuffer, unsigned int size, unsigned int length, CTC_FILE apFile )
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->CTC_fread != NULL)
		{
			return((*coreTable->CTC_fread)(apByteBuffer, size, length, apFile ));
		}
	}
	return 0;
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION (public)                                                     */ 
/*                                                                        */ 
/*    char *CTC_fgets                                                     */ 
/*                                                                        */ 
/*  AUTHOR                                                                */ 
/*                                                                        */ 
/*    Kevin Halloran                                                      */ 
/*                                                                        */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function is used to read a CR LF or LF terminated line from a  */ 
/*    file openned by a call to CTC_fopen(), including the CRLF           */
/*                                                                        */ 
/*  PARAMETERS                                                            */ 
/*                                                                        */ 
/*    char *string                   Pointer on where to place read data  */ 
/*    int count                      Number of characters to read         */ 
/*    CTC_FILE apFile                File handle returned by CTC_fopen()  */ 
/*                                                                        */ 
/*  RETURNS                                                               */ 
/*                                                                        */ 
/*    char * - NULL if error, else char *string                           */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                DESCRIPTION                   */ 
/*                                                                        */ 
/*  07-01-2003         Kevin H              Initial Version               */ 
/*                                                                        */ 
/**************************************************************************/ 
char *CTC_fgets(char *string, int count,CTC_FILE apFile)
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->CTC_fgets != NULL)
		{
			return((*coreTable->CTC_fgets)(string, count, apFile));
		}
	}
	return NULL;
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION (public)                                                     */ 
/*                                                                        */ 
/*    int CTC_fseek                                                       */ 
/*                                                                        */ 
/*  AUTHOR                                                                */ 
/*                                                                        */ 
/*    Kevin Halloran                                                      */ 
/*                                                                        */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function is used to position an open file to the desired offset*/ 
/*    from start.                                                         */
/*                                                                        */ 
/*  PARAMETERS                                                            */ 
/*                                                                        */ 
/*    CTC_FILE apFile                File handle returned by CTC_fopen()  */ 
/*    long offset                    Position from start desired          */ 
/*    int origin                     Not used                             */ 
/*                                                                        */ 
/*  RETURNS                                                               */ 
/*                                                                        */ 
/*    int - -1 if failed, else 0                                          */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                DESCRIPTION                   */ 
/*                                                                        */ 
/*  07-01-2003         Kevin H              Initial Version               */ 
/*                                                                        */ 
/**************************************************************************/ 
int CTC_fseek( CTC_FILE apFile, long offset, int origin )
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->CTC_fseek != NULL)
		{
			return((*coreTable->CTC_fseek)( apFile, offset, origin ));
		}
	}
	return -1;
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION (public)                                                     */ 
/*                                                                        */ 
/*    char *CTC_getFileName                                               */ 
/*                                                                        */ 
/*  AUTHOR                                                                */ 
/*                                                                        */ 
/*    Kevin Halloran                                                      */ 
/*                                                                        */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function is used to get a pointer to a file name using the     */ 
/*    handle returned by a call to CTC_fopen()                            */
/*                                                                        */ 
/*  PARAMETERS                                                            */ 
/*                                                                        */ 
/*    CTC_FILE apFile                File handle returned by CTC_fopen()  */ 
/*                                                                        */ 
/*  RETURNS                                                               */ 
/*                                                                        */ 
/*    char * - pointer to name of file in directory block                 */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                DESCRIPTION                   */ 
/*                                                                        */ 
/*  07-01-2003         Kevin H              Initial Version               */ 
/*                                                                        */ 
/**************************************************************************/ 
char *CTC_getFileName(CTC_FILE apFile)
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->CTC_getFileName != NULL)
		{
			return((*coreTable->CTC_getFileName)(apFile));
		}
	}
	return NULL;
}


// DIRECTORY FUNCTIONS

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION (public)                                                     */ 
/*                                                                        */ 
/*    int CTC_rmdir                                                       */ 
/*                                                                        */ 
/*  AUTHOR                                                                */ 
/*                                                                        */ 
/*    Kevin Halloran                                                      */ 
/*                                                                        */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function is used to delete an existing directory               */ 
/*                                                                        */ 
/*  PARAMETERS                                                            */ 
/*                                                                        */ 
/*    char *DirName                  Name of directory to delete          */ 
/*                                                                        */ 
/*  RETURNS                                                               */ 
/*                                                                        */ 
/*    int  - true if deleted, else false if not exist or in use           */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                DESCRIPTION                   */ 
/*                                                                        */ 
/*  07-01-2003         Kevin H              Initial Version               */ 
/*                                                                        */ 
/**************************************************************************/ 
int CTC_rmdir(char *DirName)
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->CTC_rmdir != NULL)
		{
			return((*coreTable->CTC_rmdir)(DirName));
		}
	}
	return 0;
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION (public)                                                     */ 
/*                                                                        */ 
/*    int CTC_pwd                                                         */ 
/*                                                                        */ 
/*  AUTHOR                                                                */ 
/*                                                                        */ 
/*    Kevin Halloran                                                      */ 
/*                                                                        */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function is used to retrieve the current directory path        */ 
/*                                                                        */ 
/*  PARAMETERS                                                            */ 
/*                                                                        */ 
/*    char *DirName                  Pointer of where to store current    */ 
/*                                   directory                            */ 
/*                                                                        */ 
/*  RETURNS                                                               */ 
/*                                                                        */ 
/*    int  - true if returned, else false if error                        */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                DESCRIPTION                   */ 
/*                                                                        */ 
/*  07-01-2003         Kevin H              Initial Version               */ 
/*                                                                        */ 
/**************************************************************************/ 		
int CTC_pwd(char *DirName)
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->CTC_pwd != NULL)
		{
			return((*coreTable->CTC_pwd)(DirName));
		}
	}
	return 0;
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION (public)                                                     */ 
/*                                                                        */ 
/*    int CTC_cd                                                          */ 
/*                                                                        */ 
/*  AUTHOR                                                                */ 
/*                                                                        */ 
/*    Kevin Halloran                                                      */ 
/*                                                                        */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function is used to change the active directory                */ 
/*                                                                        */ 
/*  PARAMETERS                                                            */ 
/*                                                                        */ 
/*    char *DirName                  Pointer of what directory to change  */ 
/*                                   to                                   */ 
/*                                                                        */ 
/*  RETURNS                                                               */ 
/*                                                                        */ 
/*    int  - true if directory changed, else false if failure             */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                DESCRIPTION                   */ 
/*                                                                        */ 
/*  07-01-2003         Kevin H              Initial Version               */ 
/*                                                                        */ 
/**************************************************************************/ 
int CTC_cd(char *DirName)
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->CTC_cd != NULL)
		{
			return((*coreTable->CTC_cd)(DirName));
		}
	}
	return 0;
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION (public)                                                     */ 
/*                                                                        */ 
/*    int CTC_mkdir                                                       */ 
/*                                                                        */ 
/*  AUTHOR                                                                */ 
/*                                                                        */ 
/*    Kevin Halloran                                                      */ 
/*                                                                        */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function is used to make a new directory                       */ 
/*                                                                        */ 
/*  PARAMETERS                                                            */ 
/*                                                                        */ 
/*    char *DirName                  Name of new directory to make        */ 
/*                                                                        */ 
/*  RETURNS                                                               */ 
/*                                                                        */ 
/*    int  - true if created, else false if error                         */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                DESCRIPTION                   */ 
/*                                                                        */ 
/*  07-01-2003         Kevin H              Initial Version               */ 
/*                                                                        */ 
/**************************************************************************/ 
int CTC_mkdir(char *DirName)
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->CTC_mkdir != NULL)
		{
			return((*coreTable->CTC_mkdir)(DirName));
		}
	}
	return 0;
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION (public)                                                     */ 
/*                                                                        */ 
/*    int CTC_formatDisk                                                  */ 
/*                                                                        */ 
/*  AUTHOR                                                                */ 
/*                                                                        */ 
/*    Kevin Halloran                                                      */ 
/*                                                                        */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function is used to retrieve the current directory path        */ 
/*                                                                        */ 
/*  PARAMETERS                                                            */ 
/*                                                                        */ 
/*    None                                                                */ 
/*                                                                        */ 
/*  RETURNS                                                               */ 
/*                                                                        */ 
/*    int - 0 if formated, -1 if failed                                   */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                DESCRIPTION                   */ 
/*                                                                        */ 
/*  07-01-2003         Kevin H              Initial Version               */ 
/*                                                                        */ 
/**************************************************************************/ 
int CTC_formatDisk(void)
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->CTC_formatDisk != NULL)
		{
			return((*coreTable->CTC_formatDisk)());
		}
	}
	return -1;
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION (public)                                                     */ 
/*                                                                        */ 
/*    int CTC_getFullFileName                                             */ 
/*                                                                        */ 
/*  AUTHOR                                                                */ 
/*                                                                        */ 
/*    Kevin Halloran                                                      */ 
/*                                                                        */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function is used to get a pointer to a file name using the     */ 
/*    handle returned by a call to CTC_fopen()                            */
/*                                                                        */ 
/*  PARAMETERS                                                            */ 
/*                                                                        */ 
/*    CTC_FILE apFile                File handle returned by CTC_fopen()  */ 
/*    char *name                     Pointer to where to store full path  */ 
/*                                   and file name of file referenced by  */ 
/*                                   the CTC_fopen() returned 'handle'    */ 
/*  RETURNS                                                               */ 
/*                                                                        */ 
/*    int  - true if name returned properly, false if error               */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                DESCRIPTION                   */ 
/*                                                                        */ 
/*  07-01-2003         Kevin H              Initial Version               */ 
/*                                                                        */ 
/**************************************************************************/ 
int CTC_getFullFileName(CTC_FILE handle, char *name)
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->CTC_getFullFileName != NULL)
		{
			return((*coreTable->CTC_getFullFileName)(handle, name));
		}
	}
	return 0;
}

void CTC_volumeGetInfo(unsigned long *TotalSize, unsigned long *TotalAvail, unsigned long *TotalDeleted)
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->CTC_volumeGetInfo != NULL)
		{
			((*coreTable->CTC_volumeGetInfo)(TotalSize, TotalAvail, TotalDeleted));
		}
	}
	return;
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION (public)                                                     */ 
/*                                                                        */ 
/*    int CTC_findfirst                                                   */ 
/*                                                                        */ 
/*  AUTHOR                                                                */ 
/*                                                                        */ 
/*    Kevin Halloran                                                      */ 
/*                                                                        */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function is used to walk a directory tree after an initial     */ 
/*    call to CTC_findfirst(). CTC_FIND_DATA is updated with next fileinfo*/ 
/*                                                                        */ 
/*  PARAMETERS                                                            */ 
/*                                                                        */ 
/*    char *lpFileName               Path whose tree to walk              */ 
/*    CTC_FIND_DATA *lpFindFileData  Pointer to structure to be referenced*/ 
/*                                   for all directory tree walks and to  */ 
/*                                   return file information witin        */ 
/*                                                                        */ 
/*  RETURNS                                                               */ 
/*                                                                        */ 
/*    int  - true if CTC_FIND_DATA structure updated, false if no more    */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                DESCRIPTION                   */ 
/*                                                                        */ 
/*  07-01-2003         Kevin H              Initial Version               */ 
/*                                                                        */ 
/**************************************************************************/ 
int CTC_findfirst( char *lpFileName,CTC_FIND_DATA *lpFindFileData)
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->CTC_findfirst != NULL)
		{
			return((*coreTable->CTC_findfirst)(lpFileName,lpFindFileData));
		}
	}
	return 0;
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION (public)                                                     */ 
/*                                                                        */ 
/*    int CTC_findnext                                                    */ 
/*                                                                        */ 
/*  AUTHOR                                                                */ 
/*                                                                        */ 
/*    Kevin Halloran                                                      */ 
/*                                                                        */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function is used to walk a directory tree after an initial     */ 
/*    call to CTC_findfirst(). CTC_FIND_DATA is updated with next fileinfo*/ 
/*                                                                        */ 
/*  PARAMETERS                                                            */ 
/*                                                                        */ 
/*    CTC_FIND_DATA *lpFindFileData  Pointer to structure referenced in   */ 
/*                                   initial call to CTC_findfirst()      */ 
/*                                                                        */ 
/*  RETURNS                                                               */ 
/*                                                                        */ 
/*    int  - true if CTC_FIND_DATA structure updated, false if no more    */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                DESCRIPTION                   */ 
/*                                                                        */ 
/*  07-01-2003         Kevin H              Initial Version               */ 
/*                                                                        */ 
/**************************************************************************/ 
int CTC_findnext( CTC_FIND_DATA *lpFindFileData) 
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->CTC_findnext != NULL)
		{
			return((*coreTable->CTC_findnext)(lpFindFileData));
		}
	}
	return 0;
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION (public)                                                     */ 
/*                                                                        */ 
/*    void CTC_findclose                                                  */ 
/*                                                                        */ 
/*  AUTHOR                                                                */ 
/*                                                                        */ 
/*    Kevin Halloran                                                      */ 
/*                                                                        */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function is used to close a directory tree walk                */ 
/*                                                                        */ 
/*  PARAMETERS                                                            */ 
/*                                                                        */ 
/*    CTC_FIND_DATA *lpFindFileData  Pointer to structure referenced in   */ 
/*                                   initial call to CTC_findfirst()      */ 
/*                                                                        */ 
/*  RETURNS                                                               */ 
/*                                                                        */ 
/*    None                                                                */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                DESCRIPTION                   */ 
/*                                                                        */ 
/*  07-01-2003         Kevin H              Initial Version               */ 
/*                                                                        */ 
/**************************************************************************/ 
void CTC_findclose(CTC_FIND_DATA *lpFindFileData)
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->CTC_findclose != NULL)
		{
			((*coreTable->CTC_findclose)(lpFindFileData));
		}
	}
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION (public)                                                     */ 
/*                                                                        */ 
/*    int CTC_getFileInfo                                                 */ 
/*                                                                        */ 
/*  AUTHOR                                                                */ 
/*                                                                        */ 
/*    Kevin Halloran                                                      */ 
/*                                                                        */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function is used to get information on a file that is not open */ 
/*                                                                        */ 
/*  PARAMETERS                                                            */ 
/*                                                                        */ 
/*    char *Name                     Pointer to name/path of file want    */ 
/*    CTC_FIND_DATA *lpFindFileData  Pointer to structure to return file  */ 
/*                                   information in                       */ 
/*                                                                        */ 
/*  RETURNS                                                               */ 
/*                                                                        */ 
/*    int  - true if info returned, else false (0) if failure             */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                DESCRIPTION                   */ 
/*                                                                        */ 
/*  07-01-2003         Kevin H              Initial Version               */ 
/*                                                                        */ 
/**************************************************************************/ 
int CTC_getFileInfo(char *Name, CTC_FIND_DATA *lpFindFileData)
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->CTC_getFileInfo != NULL)
		{
			return((*coreTable->CTC_getFileInfo)(Name, lpFindFileData));
		}
	}
	return 0;
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION (public)                                                     */ 
/*                                                                        */ 
/*    int CTC_remove                                                      */ 
/*                                                                        */ 
/*  AUTHOR                                                                */ 
/*                                                                        */ 
/*    Kevin Halloran                                                      */ 
/*                                                                        */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function is used to delete a file                              */ 
/*                                                                        */ 
/*  PARAMETERS                                                            */ 
/*                                                                        */ 
/*    char *filename                 Name of file to delete               */ 
/*                                                                        */ 
/*  RETURNS                                                               */ 
/*                                                                        */ 
/*    int - 0 if file deleted, -1 if not exist or in use                  */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                DESCRIPTION                   */ 
/*                                                                        */ 
/*  07-01-2003         Kevin H              Initial Version               */ 
/*                                                                        */ 
/**************************************************************************/ 
int CTC_remove(char *filename)
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->CTC_remove != NULL)
		{
			return((*coreTable->CTC_remove)(filename));
		}
	}
	return -1;
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION (public)                                                     */ 
/*                                                                        */ 
/*    int CTC_rename                                                      */ 
/*                                                                        */ 
/*  AUTHOR                                                                */ 
/*                                                                        */ 
/*    Kevin Halloran                                                      */ 
/*                                                                        */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function is used to delete a file                              */ 
/*                                                                        */ 
/*  PARAMETERS                                                            */ 
/*                                                                        */ 
/*    char *fromName                 Name of file to rename               */ 
/*    char *toName                   New name of file                     */ 
/*                                                                        */ 
/*  RETURNS                                                               */ 
/*                                                                        */ 
/*    int - 0 if file renamed, -1 if not exist or in use                  */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                DESCRIPTION                   */ 
/*                                                                        */ 
/*  11-16-2004         Kevin H              Initial Version               */ 
/*                                                                        */ 
/**************************************************************************/ 
int CTC_rename(char *fromName,char *toName)
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->CTC_rename != NULL)
		{
			return((*coreTable->CTC_rename)(fromName,toName));
		}
	}
	return -1;
}

/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION (public)                                                     */ 
/*                                                                        */ 
/*    int CTC_rendir                                                      */ 
/*                                                                        */ 
/*  AUTHOR                                                                */ 
/*                                                                        */ 
/*    Kevin Halloran                                                      */ 
/*                                                                        */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function is used to rename a directory                         */ 
/*                                                                        */ 
/*  PARAMETERS                                                            */ 
/*                                                                        */ 
/*    char *fromName                 Name of directory to rename          */ 
/*    char *toName                   New name of directory                */ 
/*                                                                        */ 
/*  RETURNS                                                               */ 
/*                                                                        */ 
/*    int - 0 if file renamed, -1 if not exist or in use                  */ 
/*                                                                        */ 
/*  RELEASE HISTORY                                                       */ 
/*                                                                        */ 
/*    DATE              NAME                DESCRIPTION                   */ 
/*                                                                        */ 
/*  11-16-2004         Kevin H              Initial Version               */ 
/*                                                                        */ 
/**************************************************************************/ 
int CTC_rendir(char *fromName,char *toName)
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->CTC_rendir != NULL)
		{
			return((*coreTable->CTC_rendir)(fromName,toName));
		}
	}
	return -1;
}
/************************* Motion Routines ********************************/
/*                                                                        */
/**************************************************************************/ 
/*                                                                        */ 
/*  FUNCTION (public)                                                     */ 
/*                                                                        */ 
/*  UINT16 Motion_GetNumMotors                                            */ 
/*                                                                        */ 
/*  DESCRIPTION                                                           */ 
/*                                                                        */ 
/*    This function is used to determine how many motors are installed    */
/*    in the system (number of axis, total)                               */ 
/*                                                                        */ 
/*  PARAMETERS                                                            */ 
/*                                                                        */ 
/*    NONE                                                                */ 
/*                                                                        */ 
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  RETURNS                                                               */ 
/*                                                                        */ 
/*    UINT16 - Number of motors installed in system, 0 if error or none   */ 
/*                                                                        */ 
/**************************************************************************/ 
UINT16 Motion_GetNumMotors(void)
{
	//	MOTION_GET_NUMMOTORS    Motion_GetNumMotors;
	if (getCoreTable() != NULL)
	{
		if (coreTable->Motion_GetNumMotors != NULL)
		{
			return((*coreTable->Motion_GetNumMotors)());
		}
	}
	return(0);
}

RETVAL Motion_Get_Attribute( UINT8 MotorNum, UINT8 Attribute, STDVAL *value)			  // NOT IMPLEMENTED
{
	//	MOTION_GET_ATTRIBUTE    Motion_Get_Attribute;
	if (getCoreTable() != NULL)
	{
		if (coreTable->Motion_Get_Attribute != NULL)
		{
			return((*coreTable->Motion_Get_Attribute)(MotorNum, Attribute, value));		 
		}
	}
	return(ERROR_NOT_DEFINED);
}

RETVAL Motion_Put_Attribute( UINT8 MotorNum, UINT8 Attribute, STDVAL value)				// NOT IMPLEMENTED
{
	//	MOTION_PUT_ATTRIBUTE    Motion_Put_Attribute;
	if (getCoreTable() != NULL)
	{
		if (coreTable->Motion_Put_Attribute != NULL)
		{
			return((*coreTable->Motion_Put_Attribute)(MotorNum, Attribute, value));
		}
	}
	return(ERROR_NOT_DEFINED);
}

RETVAL Motion_Ready( UINT8 MotorNum, STDVAL *value)										// NOT IMPLEMENTED
{
	//	MOTION_READY            Motion_Ready;
	if (getCoreTable() != NULL)
	{
		if (coreTable->Motion_Ready != NULL)
		{
			return((*coreTable->Motion_Ready)(MotorNum, value));
		}
	}
	return(ERROR_NOT_DEFINED);
}

RETVAL Motion_Simple_Command( UINT8 MotorNum, UINT8 Command)						    // NOT IMPLEMENTED
{
	//	MOTION_SIMPLE_COMMAND   Motion_Simple_Command;
	if (getCoreTable() != NULL)
	{
		if (coreTable->Motion_Simple_Command != NULL)
		{
			return((*coreTable->Motion_Simple_Command)(MotorNum, Command));
		}
	}
	return(ERROR_NOT_DEFINED);
}

/**************************************************************************/
/*                                                                        */
/*  FUNCTION (public)                                                     */
/*                                                                        */
/*    RETVAL RPC_axis_param_read        (QS4 use ONLY)                    */
/*                                                                        */
/*  DESCRIPTION                                                           */
/*                                                                        */
/*    This function allows the caller to read data of 'type' from a       */
/*    motion axis.  This call will call the same function on the motion   */
/*    controller board and block until a response or timeout occurs.      */
/*                                                                        */
/*  PARAMETERS                                                            */
/*                                                                        */
/*    TASK *task - Pointer to current Quickstep task or NULL if none.     */
/*                 Reserved for future use and not currently used.        */
/*    int axis   - Motion axis to access, with '1' being the first.       */
/*    int parameter - QS4 Motion specific parameter.                      */
/*    int type - Information on what 'value' points to:                   */
/*               VARIANT_INTEGER, VARIANT_FLOAT, VARIANT_DOUBLE,          */
/*               VARIANT_LLONGINTEGER, VARIANT_STRING, VARIANT_STRING_PTR */
/*    void *value - pointer to where to store data of 'type' to, if string*/
/*               then an array of at least VARIANT_MAX_STRING + 1         */
/*               characters is required.                                  */
/*    int *rpc_result - Pointer to where to store RPC function call return*/
/*               value from function on remote board.  Only valid if      */
/*               SUCCESS returned by function call.                       */
/*    int peeridx - Future index to remote routing table for network based*/
/*               drives.  For now keep 0 for local access.                */
/*                                                                        */
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  RETURNS                                                               */
/*                                                                        */
/*    SUCCESS if RPC call successful                                      */
/*    INVALID_PARAMETER if 'type' code is not correct                     */
/*    ERROR_NET_TIMEOUT timed out waiting for network reply               */
/*    ERROR_NET_RQST_LOST network error sending                           */
/*    ERROR_NOT_DEFINED there is no API access???                         */
/*                                                                        */
/**************************************************************************/
RETVAL RPC_axis_param_read(TASK *task, int axis, int parameter, int type, void *value, int *rpc_result, int peeridx)
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->RPC_axis_param_read != NULL)
		{
			return((*coreTable->RPC_axis_param_read)(task, axis, parameter, type, value, rpc_result, peeridx));
		}
	}
	return(ERROR_NOT_DEFINED);
}

/**************************************************************************/
/*                                                                        */
/*  FUNCTION (public)                                                     */
/*                                                                        */
/*    RETVAL RPC_axis_param_write        (QS4 use ONLY)                    */
/*                                                                        */
/*  DESCRIPTION                                                           */
/*                                                                        */
/*    This function allows the caller to write data of 'type' to a        */
/*    motion axis.  This call will call the same function on the motion   */
/*    controller board and block until a response or timeout occurs.      */
/*                                                                        */
/*  PARAMETERS                                                            */
/*                                                                        */
/*    TASK *task - Pointer to current Quickstep task or NULL if none.     */
/*                 Reserved for future use and not currently used.        */
/*    int axis   - Motion axis to access, with '1' being the first.       */
/*    int parameter - QS4 Motion specific parameter.                      */
/*    int type - Information on what 'value' points to:                   */
/*               VARIANT_INTEGER, VARIANT_FLOAT, VARIANT_DOUBLE,          */
/*               VARIANT_LLONGINTEGER, VARIANT_STRING, VARIANT_STRING_PTR */
/*    void *value - pointer to where to get data to write from.  If string*/
/*               must be null terminated and not longer than              */
/*               VARIANT_MAX_STRING characters.                           */
/*    int *rpc_result - Pointer to where to store RPC function call return*/
/*               value from function on remote board.  Only valid if      */
/*               SUCCESS returned by function call.                       */
/*    int peeridx - Future index to remote routing table for network based*/
/*               drives.  For now keep 0 for local access.                */
/*                                                                        */
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  RETURNS                                                               */
/*                                                                        */
/*    SUCCESS if RPC call successful                                      */
/*    INVALID_PARAMETER if 'type' code is not correct                     */
/*    ERROR_NET_TIMEOUT timed out waiting for network reply               */
/*    ERROR_NET_RQST_LOST network error sending                           */
/*    ERROR_NOT_DEFINED there is no API access???                         */
/*                                                                        */
/**************************************************************************/
RETVAL RPC_axis_param_write(TASK *task, int axis, int parameter, int type, void *value, int *rpc_result, int peeridx)
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->RPC_axis_param_write != NULL)
		{
			return((*coreTable->RPC_axis_param_write)(task, axis, parameter, type, value, rpc_result, peeridx));
		}
	}
	return(ERROR_NOT_DEFINED);
}

/**************************************************************************/
/*                                                                        */
/*  FUNCTION (public)                                                     */
/*                                                                        */
/*    RETVAL RPC_clear_all_MSB          (QS4 use ONLY)                    */
/*                                                                        */
/*  DESCRIPTION                                                           */
/*                                                                        */
/*    This function allows the caller to clear all stored MSB's for a     */
/*    specific motion axis.  This call will call the same function on the */
/*    motion controller board and block until response or timeout occurs. */
/*                                                                        */
/*  PARAMETERS                                                            */
/*                                                                        */
/*    TASK *task - Pointer to current Quickstep task or NULL if none.     */
/*                 Reserved for future use and not currently used.        */
/*    int axis   - Motion axis to access, with '1' being the first.       */
/*    int *rpc_result - Pointer to where to store RPC function call return*/
/*               value from function on remote board.  Only valid if      */
/*               SUCCESS returned by function call.                       */
/*    int peeridx - Future index to remote routing table for network based*/
/*               drives.  For now keep 0 for local access.                */
/*                                                                        */
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  RETURNS                                                               */
/*                                                                        */
/*    SUCCESS if RPC call successful                                      */
/*    ERROR_NET_TIMEOUT timed out waiting for network reply               */
/*    ERROR_NET_RQST_LOST network error sending                           */
/*    ERROR_NOT_DEFINED there is no API access???                         */
/*                                                                        */
/**************************************************************************/
RETVAL RPC_clear_all_MSB(TASK *task, int axis, int *rpc_result, int peeridx)
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->RPC_clear_all_MSB != NULL)
		{
			return((*coreTable->RPC_clear_all_MSB)(task, axis, rpc_result, peeridx));
		}
	}
	return(ERROR_NOT_DEFINED);
}

/**************************************************************************/
/*                                                                        */
/*  FUNCTION (public)                                                     */
/*                                                                        */
/*    RETVAL RPC_axis_start_MSB          (QS4 use ONLY)                   */
/*                                                                        */
/*  DESCRIPTION                                                           */
/*                                                                        */
/*    This function allows the caller to start a specific MSB for a       */
/*    motion axis.  This call will call the same function on the motion   */
/*    controller board and block until a response or timeout occurs.      */
/*                                                                        */
/*  PARAMETERS                                                            */
/*                                                                        */
/*    TASK *task - Pointer to current Quickstep task or NULL if none.     */
/*                 Reserved for future use and not currently used.        */
/*    int axis   - Motion axis to access, with '1' being the first.       */
/*    int msb_number - Desired MSB to start.                              */
/*    int *rpc_result - Pointer to where to store RPC function call return*/
/*               value from function on remote board.  Only valid if      */
/*               SUCCESS returned by function call.                       */
/*    int peeridx - Future index to remote routing table for network based*/
/*               drives.  For now keep 0 for local access.                */
/*                                                                        */
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  RETURNS                                                               */
/*                                                                        */
/*    SUCCESS if RPC call successful                                      */
/*    ERROR_NET_TIMEOUT timed out waiting for network reply               */
/*    ERROR_NET_RQST_LOST network error sending                           */
/*    ERROR_NOT_DEFINED there is no API access???                         */
/*                                                                        */
/**************************************************************************/
RETVAL RPC_axis_start_MSB(TASK *task, int axis, int msb_number, int *rpc_result, int peeridx)
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->RPC_axis_start_MSB != NULL)
		{
			return((*coreTable->RPC_axis_start_MSB)(task, axis, msb_number, rpc_result, peeridx));
		}
	}
	return(ERROR_NOT_DEFINED);
}

/**************************************************************************/
/*                                                                        */
/*  FUNCTION (public)                                                     */
/*                                                                        */
/*    RETVAL RPC_axis_stop_MSB          (QS4 use ONLY)                    */
/*                                                                        */
/*  DESCRIPTION                                                           */
/*                                                                        */
/*    This function allows the caller to stop a specific MSB for a        */
/*    motion axis.  This call will call the same function on the motion   */
/*    controller board and block until a response or timeout occurs.      */
/*                                                                        */
/*  PARAMETERS                                                            */
/*                                                                        */
/*    TASK *task - Pointer to current Quickstep task or NULL if none.     */
/*                 Reserved for future use and not currently used.        */
/*    int axis   - Motion axis to access, with '1' being the first.       */
/*    int msb_number - Desired MSB to stop.                               */
/*    int *rpc_result - Pointer to where to store RPC function call return*/
/*               value from function on remote board.  Only valid if      */
/*               SUCCESS returned by function call.                       */
/*    int peeridx - Future index to remote routing table for network based*/
/*               drives.  For now keep 0 for local access.                */
/*                                                                        */
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  RETURNS                                                               */
/*                                                                        */
/*    SUCCESS if RPC call successful                                      */
/*    ERROR_NET_TIMEOUT timed out waiting for network reply               */
/*    ERROR_NET_RQST_LOST network error sending                           */
/*    ERROR_NOT_DEFINED there is no API access???                         */
/*                                                                        */
/**************************************************************************/
RETVAL RPC_axis_stop_MSB(TASK *task, int axis, int msb_number, int *rpc_result, int peeridx)
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->RPC_axis_stop_MSB != NULL)
		{
			return((*coreTable->RPC_axis_stop_MSB)(task, axis, msb_number, rpc_result, peeridx));
		}
	}
	return(ERROR_NOT_DEFINED);
}

/**************************************************************************/
/*                                                                        */
/*  FUNCTION (public)                                                     */
/*                                                                        */
/*    RETVAL RPC_axis_status          (QS4 use ONLY)                      */
/*                                                                        */
/*  DESCRIPTION                                                           */
/*                                                                        */
/*    This function allows the caller to get the status of a              */
/*    motion axis.  This call will call the same function on the motion   */
/*    controller board and block until a response or timeout occurs.      */
/*                                                                        */
/*  PARAMETERS                                                            */
/*                                                                        */
/*    TASK *task - Pointer to current Quickstep task or NULL if none.     */
/*                 Reserved for future use and not currently used.        */
/*    int axis   - Motion axis to access, with '1' being the first.       */
/*    int *status - Pointer to where to store axis status.                */
/*    int *rpc_result - Pointer to where to store RPC function call return*/
/*               value from function on remote board.  Only valid if      */
/*               SUCCESS returned by function call.                       */
/*    int peeridx - Future index to remote routing table for network based*/
/*               drives.  For now keep 0 for local access.                */
/*                                                                        */
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  RETURNS                                                               */
/*                                                                        */
/*    SUCCESS if RPC call successful                                      */
/*    ERROR_NET_TIMEOUT timed out waiting for network reply               */
/*    ERROR_NET_RQST_LOST network error sending                           */
/*    ERROR_NOT_DEFINED there is no API access???                         */
/*                                                                        */
/**************************************************************************/
RETVAL RPC_axis_status(TASK *task, int axis, int *status, int *rpc_result, int peeridx)
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->RPC_axis_status != NULL)
		{
			return((*coreTable->RPC_axis_status)(task, axis, status, rpc_result, peeridx));
		}
	}
	return(ERROR_NOT_DEFINED);
}

/**************************************************************************/
/*                                                                        */
/*  FUNCTION (public)                                                     */
/*                                                                        */
/*    RETVAL RPC_load_MSB           (QS4 use ONLY)                        */
/*                                                                        */
/*  DESCRIPTION                                                           */
/*                                                                        */
/*    This function allows the caller to get the status of a              */
/*    motion axis.  This call will call the same function on the motion   */
/*    controller board and block until a response or timeout occurs.      */
/*                                                                        */
/*  PARAMETERS                                                            */
/*                                                                        */
/*    TASK *task - Pointer to current Quickstep task or NULL if none.     */
/*                 Reserved for future use and not currently used.        */
/*    int axis   - Motion axis to access, with '1' being the first.       */
/*    int msb_number - Desired MSB to load.                               */
/*    void *msb - Pointer to data block to transfer.                      */
/*    int size -  Number of bytes to transfer.                            */
/*    int *rpc_result - Pointer to where to store RPC function call return*/
/*                value from function on remote board.  Only valid if     */
/*                SUCCESS returned by function call.                      */
/*    int peeridx - Future index to remote routing table for network based*/
/*                drives.  For now keep 0 for local access.               */
/*                                                                        */
/*  CALLS                                                                 */ 
/*    Quickstep OS virtual table function pointer                         */ 
/*                                                                        */ 
/*  RETURNS                                                               */
/*                                                                        */
/*    SUCCESS if RPC call successful                                      */
/*    ERROR_NET_TIMEOUT timed out waiting for network reply               */
/*    ERROR_NET_RQST_LOST network error sending                           */
/*    ERROR_NOT_DEFINED there is no API access???                         */
/*                                                                        */
/**************************************************************************/
RETVAL RPC_load_MSB(TASK *task, int axis, int msb_number, void* msb, int size, int *rpc_result, int peeridx)
{
	if (getCoreTable() != NULL)
	{
		if (coreTable->RPC_load_MSB != NULL)
		{
			return((*coreTable->RPC_load_MSB)(task, axis, msb_number, msb, size, rpc_result, peeridx));
		}
	}
	return(ERROR_NOT_DEFINED);
}

/*
***********************************************************
Function access table
***********************************************************
*/
void userShutdown()
{
extern void _systemCleanUp();
void userCleanUp();

logEvent(EVENT_DEBUG,8000);
	// Place any user cleanup here...
	userCleanUp();
	// now shutdown this environment so can allow reload...
	_systemCleanUp();
	// now return to Quickstep, there will be a 2 second delay before starting the load...
}

// Note that any extern's not defined below will automatically be set to 0 by the linker
// as non-entry...
extern int  main(void);
extern int  taskLoop(SYSMODE state);
/*
extern void timerTic(void);
extern int  taskLoop(SYSMODE state);

extern int  superTaskLoop(SYSMODE state);
extern int  serviceLoopHook(SYSMODE state);
extern void endAllTasks(void);
extern void taskStart(TASK *task);
extern void taskEnd(TASK *task);
void  pidControl(void);					
void *QS4ObjectTable(int *size);
*/
extern void  taskStart(TASK *task);
extern void taskEnd(TASK *task);

// Below is main table referenced by the Quickstep Operating System, tread carefully when changing
// as it must match that of the OS...  NOTE:  Change version and entries number in table!!!
const VTABLE_USER localUserCommand __attribute__ ((section (".ftable"))) = {
    VALID_USER_SIGNATURE,       // signature
    0x0001000C,                 // version 00.01, 12 entries
    main,                       // initialize
    NULL,                   	// function unused
    NULL,                   	// timerTic;
    taskLoop,                   // taskLoop;
    NULL,                       // superTaskLoop;
    NULL,                       // serviceLoopHook;
    NULL,                     	// endAllTasks;
	userShutdown,				// userShutdown;
	taskStart,					// newTaskStarted;  (called if non-null when task first created, allocate task user memory)
	taskEnd,					// taskEnded;  (called when a task is ending, free user task memory
	NULL,						// pidControl; invoked periodically, after all analog inputs are scanned
	NULL						// QS4ObjectTable;	invoked to access QS4 object table pointer and size.
    };

